@@ -17,7 +17,6 @@
 
 //! Command within CLI
 
-use crate::context::Context;
 use crate::functions::{display_all_functions, Function};
 use crate::print_format::PrintFormat;
 use crate::print_options::PrintOptions;
@@ -26,6 +25,7 @@ use datafusion::arrow::array::{ArrayRef, StringArray};
 use datafusion::arrow::datatypes::{DataType, Field, Schema};
 use datafusion::arrow::record_batch::RecordBatch;
 use datafusion::error::{DataFusionError, Result};
+use datafusion::prelude::SessionContext;
 use std::str::FromStr;
 use std::sync::Arc;
 use std::time::Instant;
@@ -50,7 +50,7 @@ pub enum OutputFormat {
 impl Command {
     pub async fn execute(
         &self,
-        ctx: &mut Context,
+        ctx: &mut SessionContext,
         print_options: &mut PrintOptions,
     ) -> Result<()> {
         let now = Instant::now();
