@@ -19,11 +19,11 @@
 
 use crate::{
     command::{Command, OutputFormat},
-    context::Context,
     helper::CliHelper,
     print_options::PrintOptions,
 };
 use datafusion::error::Result;
+use datafusion::prelude::SessionContext;
 use rustyline::error::ReadlineError;
 use rustyline::Editor;
 use std::fs::File;
@@ -33,7 +33,7 @@ use std::time::Instant;
 
 /// run and execute SQL statements and commands from a file, against a context with the given print options
 pub async fn exec_from_lines(
-    ctx: &mut Context,
+    ctx: &mut SessionContext,
     reader: &mut BufReader<File>,
     print_options: &PrintOptions,
 ) {
@@ -74,7 +74,7 @@ pub async fn exec_from_lines(
 
 pub async fn exec_from_files(
     files: Vec<String>,
-    ctx: &mut Context,
+    ctx: &mut SessionContext,
     print_options: &PrintOptions,
 ) {
     let files = files
@@ -88,7 +88,7 @@ pub async fn exec_from_files(
 }
 
 /// run and execute SQL statements and commands against a context with the given print options
-pub async fn exec_from_repl(ctx: &mut Context, print_options: &mut PrintOptions) {
+pub async fn exec_from_repl(ctx: &mut SessionContext, print_options: &mut PrintOptions) {
     let mut rl = Editor::<CliHelper>::new();
     rl.set_helper(Some(CliHelper::default()));
     rl.load_history(".history").ok();
@@ -157,7 +157,7 @@ pub async fn exec_from_repl(ctx: &mut Context, print_options: &mut PrintOptions)
 }
 
 async fn exec_and_print(
-    ctx: &mut Context,
+    ctx: &mut SessionContext,
     print_options: &PrintOptions,
     sql: String,
 ) -> Result<()> {
