@@ -265,7 +265,7 @@ mod tests {
 
     use crate::error::Result;
     use crate::logical_plan::Operator;
-    use crate::physical_plan::aggregates::AggregateExec;
+    use crate::physical_plan::aggregates::{AggregateExec, PhysicalGroupBy};
     use crate::physical_plan::coalesce_partitions::CoalescePartitionsExec;
     use crate::physical_plan::common;
     use crate::physical_plan::expressions::Count;
@@ -276,8 +276,8 @@ mod tests {
     /// Mock data using a MemoryExec which has an exact count statistic
     fn mock_data() -> Result<Arc<MemoryExec>> {
         let schema = Arc::new(Schema::new(vec![
-            Field::new("a", DataType::Int32, false),
-            Field::new("b", DataType::Int32, false),
+            Field::new("a", DataType::Int32, true),
+            Field::new("b", DataType::Int32, true),
         ]));
 
         let batch = RecordBatch::try_new(
@@ -407,7 +407,7 @@ mod tests {
 
         let partial_agg = AggregateExec::try_new(
             AggregateMode::Partial,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             source,
             Arc::clone(&schema),
@@ -415,7 +415,7 @@ mod tests {
 
         let final_agg = AggregateExec::try_new(
             AggregateMode::Final,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             Arc::new(partial_agg),
             Arc::clone(&schema),
@@ -435,7 +435,7 @@ mod tests {
 
         let partial_agg = AggregateExec::try_new(
             AggregateMode::Partial,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             source,
             Arc::clone(&schema),
@@ -443,7 +443,7 @@ mod tests {
 
         let final_agg = AggregateExec::try_new(
             AggregateMode::Final,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             Arc::new(partial_agg),
             Arc::clone(&schema),
@@ -462,7 +462,7 @@ mod tests {
 
         let partial_agg = AggregateExec::try_new(
             AggregateMode::Partial,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             source,
             Arc::clone(&schema),
@@ -473,7 +473,7 @@ mod tests {
 
         let final_agg = AggregateExec::try_new(
             AggregateMode::Final,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             Arc::new(coalesce),
             Arc::clone(&schema),
@@ -492,7 +492,7 @@ mod tests {
 
         let partial_agg = AggregateExec::try_new(
             AggregateMode::Partial,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             source,
             Arc::clone(&schema),
@@ -503,7 +503,7 @@ mod tests {
 
         let final_agg = AggregateExec::try_new(
             AggregateMode::Final,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             Arc::new(coalesce),
             Arc::clone(&schema),
@@ -525,7 +525,7 @@ mod tests {
             expressions::binary(
                 expressions::col("a", &schema)?,
                 Operator::Gt,
-                expressions::lit(ScalarValue::from(1u32)),
+                expressions::lit(1u32),
                 &schema,
             )?,
             source,
@@ -533,7 +533,7 @@ mod tests {
 
         let partial_agg = AggregateExec::try_new(
             AggregateMode::Partial,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             filter,
             Arc::clone(&schema),
@@ -541,7 +541,7 @@ mod tests {
 
         let final_agg = AggregateExec::try_new(
             AggregateMode::Final,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             Arc::new(partial_agg),
             Arc::clone(&schema),
@@ -568,7 +568,7 @@ mod tests {
             expressions::binary(
                 expressions::col("a", &schema)?,
                 Operator::Gt,
-                expressions::lit(ScalarValue::from(1u32)),
+                expressions::lit(1u32),
                 &schema,
             )?,
             source,
@@ -576,7 +576,7 @@ mod tests {
 
         let partial_agg = AggregateExec::try_new(
             AggregateMode::Partial,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             filter,
             Arc::clone(&schema),
@@ -584,7 +584,7 @@ mod tests {
 
         let final_agg = AggregateExec::try_new(
             AggregateMode::Final,
-            vec![],
+            PhysicalGroupBy::default(),
             vec![agg.count_expr()],
             Arc::new(partial_agg),
             Arc::clone(&schema),
