@@ -18,42 +18,40 @@
 //! Execution plan for reading Parquet files
 
 use fmt::Debug;
-use std::collections::VecDeque;
 use std::fmt;
 use std::fs;
-use std::path::Path;
-use std::pin::Pin;
+use std::ops::Range;
 use std::sync::Arc;
-use std::task::{Context, Poll};
 use std::{any::Any, convert::TryInto};
 
 use arrow::{
     array::ArrayRef,
     datatypes::{Schema, SchemaRef},
-    error::{ArrowError, Result as ArrowResult},
-    record_batch::RecordBatch,
+    error::ArrowError,
 };
-use futures::{Stream, StreamExt, TryStreamExt};
+use bytes::Bytes;
+use futures::future::BoxFuture;
+use futures::{FutureExt, StreamExt, TryFutureExt, TryStreamExt};
 use log::debug;
-use parquet::arrow::{
-    arrow_reader::ParquetRecordBatchReader, ArrowReader, ArrowWriter,
-    ParquetFileArrowReader, ProjectionMask,
-};
-use parquet::file::reader::FileReader;
+use object_store::{ObjectMeta, ObjectStore};
+use parquet::arrow::async_reader::AsyncFileReader;
+use parquet::arrow::{ArrowWriter, ParquetRecordBatchStreamBuilder, ProjectionMask};
+use parquet::errors::ParquetError;
 use parquet::file::{
-    metadata::RowGroupMetaData, properties::WriterProperties,
-    reader::SerializedFileReader, serialized_reader::ReadOptionsBuilder,
+    metadata::{ParquetMetaData, RowGroupMetaData},
+    properties::WriterProperties,
     statistics::Statistics as ParquetStatistics,
 };
 
 use datafusion_common::Column;
-use datafusion_data_access::object_store::ObjectStore;
 use datafusion_expr::Expr;
 
-use crate::physical_plan::metrics::BaselineMetrics;
-use crate::physical_plan::stream::RecordBatchReceiverStream;
+use crate::datasource::file_format::parquet::fetch_parquet_metadata;
+use crate::datasource::listing::FileRange;
+use crate::physical_plan::file_format::file_stream::{
+    FileStream, FormatReader, ReaderFuture,
+};
 use crate::{
-    datasource::{file_format::parquet::ChunkObjectReader, listing::PartitionedFile},
     error::{DataFusionError, Result},
     execution::context::{SessionState, TaskContext},
     physical_optimizer::pruning::{PruningPredicate, PruningStatistics},
@@ -61,14 +59,12 @@ use crate::{
         expressions::PhysicalSortExpr,
         file_format::{FileScanConfig, SchemaAdapter},
         metrics::{self, ExecutionPlanMetricsSet, MetricBuilder, MetricsSet},
-        DisplayFormatType, ExecutionPlan, Partitioning, RecordBatchStream,
-        SendableRecordBatchStream, Statistics,
+        DisplayFormatType, ExecutionPlan, Partitioning, SendableRecordBatchStream,
+        Statistics,
     },
     scalar::ScalarValue,
 };
 
-use super::PartitionColumnProjector;
-
 /// Execution plan for scanning one or more Parquet partitions
 #[derive(Debug, Clone)]
 pub struct ParquetExec {
@@ -94,7 +90,6 @@ struct ParquetFileMetrics {
 
 impl ParquetExec {
     /// Create a new Parquet reader execution plan provided file list and schema.
-    /// Even if `limit` is set, ParquetExec rounds up the number of records to the next `batch_size`.
     pub fn new(base_config: FileScanConfig, predicate: Option<Expr>) -> Self {
         debug!("Creating ParquetExec, files: {:?}, projection {:?}, predicate: {:?}, limit: {:?}",
         base_config.file_groups, base_config.projection, predicate, base_config.limit);
@@ -210,52 +205,20 @@ impl ExecutionPlan for ParquetExec {
             Some(proj) => proj,
             None => (0..self.base_config.file_schema.fields().len()).collect(),
         };
-        let partition_col_proj = PartitionColumnProjector::new(
-            Arc::clone(&self.projected_schema),
-            &self.base_config.table_partition_cols,
-        );
 
-        let object_store = context
-            .runtime_env()
-            .object_store(&self.base_config.object_store_url)?;
-
-        let stream = ParquetExecStream {
-            error: false,
+        let opener = ParquetOpener {
             partition_index,
-            metrics: self.metrics.clone(),
-            object_store,
+            projection: Arc::from(projection),
+            batch_size: context.session_config().batch_size(),
             pruning_predicate: self.pruning_predicate.clone(),
-            batch_size: context.session_config().batch_size,
-            schema: self.projected_schema.clone(),
-            projection,
-            remaining_rows: self.base_config.limit,
-            reader: None,
-            files: self.base_config.file_groups[partition_index].clone().into(),
-            projector: partition_col_proj,
-            adapter: SchemaAdapter::new(self.base_config.file_schema.clone()),
-            baseline_metrics: BaselineMetrics::new(&self.metrics, partition_index),
+            table_schema: self.base_config.file_schema.clone(),
+            metrics: self.metrics.clone(),
         };
 
-        // Use spawn_blocking only if running from a tokio context (#2201)
-        match tokio::runtime::Handle::try_current() {
-            Ok(handle) => {
-                let (response_tx, response_rx) = tokio::sync::mpsc::channel(2);
-                let schema = stream.schema();
-                let join_handle = handle.spawn_blocking(move || {
-                    for result in stream {
-                        if response_tx.blocking_send(result).is_err() {
-                            break;
-                        }
-                    }
-                });
-                Ok(RecordBatchReceiverStream::create(
-                    &schema,
-                    response_rx,
-                    join_handle,
-                ))
-            }
-            Err(_) => Ok(Box::pin(stream)),
-        }
+        let stream =
+            FileStream::new(&self.base_config, partition_index, context, opener)?;
+
+        Ok(Box::pin(stream))
     }
 
     fn fmt_as(
@@ -296,164 +259,110 @@ impl ExecutionPlan for ParquetExec {
     }
 }
 
-/// Implements [`RecordBatchStream`] for a collection of [`PartitionedFile`]
-///
-/// NB: This will perform blocking IO synchronously without yielding which may
-/// be problematic in certain contexts (e.g. a tokio runtime that also performs
-/// network IO)
-struct ParquetExecStream {
-    error: bool,
+/// Implements [`FormatReader`] for a parquet file
+struct ParquetOpener {
     partition_index: usize,
-    metrics: ExecutionPlanMetricsSet,
-    object_store: Arc<dyn ObjectStore>,
-    pruning_predicate: Option<PruningPredicate>,
+    projection: Arc<[usize]>,
     batch_size: usize,
-    schema: SchemaRef,
-    projection: Vec<usize>,
-    remaining_rows: Option<usize>,
-    reader: Option<(ParquetRecordBatchReader, PartitionedFile)>,
-    files: VecDeque<PartitionedFile>,
-    projector: PartitionColumnProjector,
-    adapter: SchemaAdapter,
-    baseline_metrics: BaselineMetrics,
+    pruning_predicate: Option<PruningPredicate>,
+    table_schema: SchemaRef,
+    metrics: ExecutionPlanMetricsSet,
 }
 
-impl ParquetExecStream {
-    fn create_reader(
-        &mut self,
-        file: &PartitionedFile,
-    ) -> Result<ParquetRecordBatchReader> {
-        let file_metrics = ParquetFileMetrics::new(
+impl FormatReader for ParquetOpener {
+    fn open(
+        &self,
+        store: Arc<dyn ObjectStore>,
+        meta: ObjectMeta,
+        range: Option<FileRange>,
+    ) -> ReaderFuture {
+        let metrics = ParquetFileMetrics::new(
             self.partition_index,
-            file.file_meta.path(),
+            meta.location.as_ref(),
             &self.metrics,
         );
-        let bytes_scanned = file_metrics.bytes_scanned.clone();
-        let object_reader = self
-            .object_store
-            .file_reader(file.file_meta.sized_file.clone())?;
 
-        let mut opt = ReadOptionsBuilder::new();
-        if let Some(pruning_predicate) = &self.pruning_predicate {
-            opt = opt.with_predicate(build_row_group_predicate(
-                pruning_predicate,
-                file_metrics,
-            ));
-        }
-        if let Some(range) = &file.range {
-            assert!(
-                range.start >= 0 && range.end > 0 && range.end > range.start,
-                "invalid range specified: {:?}",
-                range
+        let reader = ParquetFileReader {
+            store,
+            meta,
+            metrics: metrics.clone(),
+        };
+
+        let schema_adapter = SchemaAdapter::new(self.table_schema.clone());
+        let batch_size = self.batch_size;
+        let projection = self.projection.clone();
+        let pruning_predicate = self.pruning_predicate.clone();
+
+        Box::pin(async move {
+            let builder = ParquetRecordBatchStreamBuilder::new(reader).await?;
+            let adapted_projections =
+                schema_adapter.map_projections(builder.schema(), &projection)?;
+
+            let mask = ProjectionMask::roots(
+                builder.parquet_schema(),
+                adapted_projections.iter().cloned(),
             );
-            opt = opt.with_range(range.start, range.end);
-        }
 
-        let file_reader = SerializedFileReader::new_with_options(
-            ChunkObjectReader {
-                object_reader,
-                bytes_scanned: Some(bytes_scanned),
-            },
-            opt.build(),
-        )?;
+            let groups = builder.metadata().row_groups();
+            let row_groups = prune_row_groups(groups, range, pruning_predicate, &metrics);
 
-        let file_metadata = file_reader.metadata().file_metadata();
-        let parquet_schema = file_metadata.schema_descr_ptr();
+            let stream = builder
+                .with_projection(mask)
+                .with_batch_size(batch_size)
+                .with_row_groups(row_groups)
+                .build()?;
 
-        let mut arrow_reader = ParquetFileArrowReader::new(Arc::new(file_reader));
-        let arrow_schema = arrow_reader.get_schema()?;
-
-        let adapted_projections = self
-            .adapter
-            .map_projections(&arrow_schema, &self.projection)?;
-
-        let mask = ProjectionMask::roots(&parquet_schema, adapted_projections);
-        let reader = arrow_reader.get_record_reader_by_columns(mask, self.batch_size)?;
-
-        Ok(reader)
-    }
-}
-
-impl Iterator for ParquetExecStream {
-    type Item = ArrowResult<RecordBatch>;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        let cloned_time = self.baseline_metrics.elapsed_compute().clone();
-        // records time on drop
-        let _timer = cloned_time.timer();
-
-        if self.error || matches!(self.remaining_rows, Some(0)) {
-            return None;
-        }
-
-        // TODO: Split this out into separate operators (#2079)
-        loop {
-            let (reader, file) = match self.reader.as_mut() {
-                Some(current) => current,
-                None => match self.files.pop_front() {
-                    None => return None,
-                    Some(file) => match self.create_reader(&file) {
-                        Ok(reader) => self.reader.insert((reader, file)),
-                        Err(e) => {
-                            self.error = true;
-                            return Some(Err(ArrowError::ExternalError(Box::new(e))));
-                        }
-                    },
-                },
-            };
-
-            let result = reader.next().map(|result| {
-                result
-                    .and_then(|batch| {
-                        self.adapter
-                            .adapt_batch(batch, &self.projection)
-                            .map_err(|e| ArrowError::ExternalError(Box::new(e)))
+            let adapted = stream
+                .map_err(|e| ArrowError::ExternalError(Box::new(e)))
+                .map(move |maybe_batch| {
+                    maybe_batch.and_then(|b| {
+                        schema_adapter
+                            .adapt_batch(b, &projection)
+                            .map_err(Into::into)
                     })
-                    .and_then(|batch| {
-                        self.projector.project(batch, &file.partition_values)
-                    })
-            });
+                });
 
-            let result = match result {
-                Some(result) => result,
-                None => {
-                    self.reader = None;
-                    continue;
-                }
-            };
-
-            match (&result, self.remaining_rows.as_mut()) {
-                (Ok(batch), Some(remaining_rows)) => {
-                    *remaining_rows = remaining_rows.saturating_sub(batch.num_rows());
-                }
-                _ => self.error = result.is_err(),
-            }
-
-            //record output rows in parquetExec
-            if let Ok(batch) = &result {
-                self.baseline_metrics.record_output(batch.num_rows());
-            }
-
-            return Some(result);
-        }
+            Ok(adapted.boxed())
+        })
     }
 }
 
-impl Stream for ParquetExecStream {
-    type Item = ArrowResult<RecordBatch>;
-
-    fn poll_next(
-        mut self: Pin<&mut Self>,
-        _cx: &mut Context<'_>,
-    ) -> Poll<Option<Self::Item>> {
-        let poll = Poll::Ready(Iterator::next(&mut *self));
-        self.baseline_metrics.record_poll(poll)
-    }
+/// Implements [`AsyncFileReader`] for a parquet file in object storage
+struct ParquetFileReader {
+    store: Arc<dyn ObjectStore>,
+    meta: ObjectMeta,
+    metrics: ParquetFileMetrics,
 }
 
-impl RecordBatchStream for ParquetExecStream {
-    fn schema(&self) -> SchemaRef {
-        self.schema.clone()
+impl AsyncFileReader for ParquetFileReader {
+    fn get_bytes(
+        &mut self,
+        range: Range<usize>,
+    ) -> BoxFuture<'_, parquet::errors::Result<Bytes>> {
+        self.metrics.bytes_scanned.add(range.end - range.start);
+
+        self.store
+            .get_range(&self.meta.location, range)
+            .map_err(|e| {
+                ParquetError::General(format!("AsyncChunkReader::get_bytes error: {}", e))
+            })
+            .boxed()
+    }
+
+    fn get_metadata(
+        &mut self,
+    ) -> BoxFuture<'_, parquet::errors::Result<Arc<ParquetMetaData>>> {
+        Box::pin(async move {
+            let metadata = fetch_parquet_metadata(self.store.as_ref(), &self.meta)
+                .await
+                .map_err(|e| {
+                    ParquetError::General(format!(
+                        "AsyncChunkReader::get_metadata error: {}",
+                        e
+                    ))
+                })?;
+            Ok(Arc::new(metadata))
+        })
     }
 }
 
@@ -556,36 +465,47 @@ impl<'a> PruningStatistics for RowGroupPruningStatistics<'a> {
     }
 }
 
-fn build_row_group_predicate(
-    pruning_predicate: &PruningPredicate,
-    metrics: ParquetFileMetrics,
-) -> Box<dyn FnMut(&RowGroupMetaData, usize) -> bool> {
-    let pruning_predicate = pruning_predicate.clone();
-    Box::new(
-        move |row_group_metadata: &RowGroupMetaData, _i: usize| -> bool {
-            let parquet_schema = pruning_predicate.schema().as_ref();
+fn prune_row_groups(
+    groups: &[RowGroupMetaData],
+    range: Option<FileRange>,
+    predicate: Option<PruningPredicate>,
+    metrics: &ParquetFileMetrics,
+) -> Vec<usize> {
+    // TODO: Columnar pruning
+    let mut filtered = Vec::with_capacity(groups.len());
+    for (idx, metadata) in groups.iter().enumerate() {
+        if let Some(range) = &range {
+            let offset = metadata.column(0).file_offset();
+            if offset < range.start || offset >= range.end {
+                continue;
+            }
+        }
+
+        if let Some(predicate) = &predicate {
             let pruning_stats = RowGroupPruningStatistics {
-                row_group_metadata,
-                parquet_schema,
+                row_group_metadata: metadata,
+                parquet_schema: predicate.schema().as_ref(),
             };
-            let predicate_values = pruning_predicate.prune(&pruning_stats);
-            match predicate_values {
+            match predicate.prune(&pruning_stats) {
                 Ok(values) => {
                     // NB: false means don't scan row group
-                    let num_pruned = values.iter().filter(|&v| !*v).count();
-                    metrics.row_groups_pruned.add(num_pruned);
-                    values[0]
+                    if !values[0] {
+                        metrics.row_groups_pruned.add(1);
+                        continue;
+                    }
                 }
                 // stats filter array could not be built
                 // return a closure which will not filter out any row groups
                 Err(e) => {
                     debug!("Error evaluating row group predicate values {}", e);
                     metrics.predicate_evaluation_errors.add(1);
-                    true
                 }
             }
-        },
-    )
+        }
+
+        filtered.push(idx)
+    }
+    filtered
 }
 
 /// Executes a query and writes the results to a partitioned Parquet file.
@@ -597,7 +517,7 @@ pub async fn plan_to_parquet(
 ) -> Result<()> {
     let path = path.as_ref();
     // create directory to contain the Parquet files (one per partition)
-    let fs_path = Path::new(path);
+    let fs_path = std::path::Path::new(path);
     match fs::create_dir(fs_path) {
         Ok(()) => {
             let mut tasks = vec![];
@@ -635,9 +555,6 @@ pub async fn plan_to_parquet(
 mod tests {
     use crate::{
         assert_batches_sorted_eq, assert_contains,
-        datafusion_data_access::{
-            object_store::local::LocalFileSystem, FileMeta, SizedFile,
-        },
         datasource::file_format::{parquet::ParquetFormat, FileFormat},
         physical_plan::collect,
     };
@@ -645,18 +562,23 @@ mod tests {
     use super::*;
     use crate::datasource::file_format::parquet::test_util::store_parquet;
     use crate::datasource::file_format::test_util::scan_format;
-    use crate::datasource::listing::FileRange;
+    use crate::datasource::listing::{FileRange, PartitionedFile};
     use crate::datasource::object_store::ObjectStoreUrl;
     use crate::execution::options::CsvReadOptions;
     use crate::prelude::{ParquetReadOptions, SessionConfig, SessionContext};
+    use crate::test::object_store::local_unpartitioned_file;
     use arrow::array::Float32Array;
+    use arrow::record_batch::RecordBatch;
     use arrow::{
         array::{Int64Array, Int8Array, StringArray},
         datatypes::{DataType, Field},
     };
-    use datafusion_data_access::object_store::local::local_unpartitioned_file;
+    use chrono::{TimeZone, Utc};
     use datafusion_expr::{col, lit};
     use futures::StreamExt;
+    use object_store::local::LocalFileSystem;
+    use object_store::path::Path;
+    use object_store::ObjectMeta;
     use parquet::{
         basic::Type as PhysicalType,
         file::{metadata::RowGroupMetaData, statistics::Statistics as ParquetStatistics},
@@ -1057,9 +979,9 @@ mod tests {
 
     #[tokio::test]
     async fn parquet_exec_with_range() -> Result<()> {
-        fn file_range(meta: &FileMeta, start: i64, end: i64) -> PartitionedFile {
+        fn file_range(meta: &ObjectMeta, start: i64, end: i64) -> PartitionedFile {
             PartitionedFile {
-                file_meta: meta.clone(),
+                object_meta: meta.clone(),
                 partition_values: vec![],
                 range: Some(FileRange { start, end }),
             }
@@ -1102,7 +1024,7 @@ mod tests {
 
         let meta = local_unpartitioned_file(filename);
 
-        let store = Arc::new(LocalFileSystem {}) as _;
+        let store = Arc::new(LocalFileSystem::new()) as _;
         let file_schema = ParquetFormat::default()
             .infer_schema(&store, &[meta.clone()])
             .await?;
@@ -1156,7 +1078,7 @@ mod tests {
             .unwrap();
 
         let partitioned_file = PartitionedFile {
-            file_meta: meta,
+            object_meta: meta,
             partition_values: vec![
                 ScalarValue::Utf8(Some("2021".to_owned())),
                 ScalarValue::Utf8(Some("10".to_owned())),
@@ -1212,13 +1134,15 @@ mod tests {
     async fn parquet_exec_with_error() -> Result<()> {
         let session_ctx = SessionContext::new();
         let task_ctx = session_ctx.task_ctx();
+        let location = Path::from_filesystem_path(".")
+            .unwrap()
+            .child("invalid.parquet");
+
         let partitioned_file = PartitionedFile {
-            file_meta: FileMeta {
-                sized_file: SizedFile {
-                    size: 1337,
-                    path: "invalid".into(),
-                },
-                last_modified: None,
+            object_meta: ObjectMeta {
+                location,
+                last_modified: Utc.timestamp_nanos(0),
+                size: 1337,
             },
             partition_values: vec![],
             range: None,
@@ -1240,10 +1164,7 @@ mod tests {
         let mut results = parquet_exec.execute(0, task_ctx)?;
         let batch = results.next().await.unwrap();
         // invalid file should produce an error to that effect
-        assert_contains!(
-            batch.unwrap_err().to_string(),
-            "External error: Parquet error: Arrow: IO error"
-        );
+        assert_contains!(batch.unwrap_err().to_string(), "invalid.parquet not found");
         assert!(results.next().await.is_none());
 
         Ok(())
@@ -1255,12 +1176,13 @@ mod tests {
     }
 
     #[test]
-    fn row_group_pruning_predicate_simple_expr() -> Result<()> {
+    fn row_group_pruning_predicate_simple_expr() {
         use datafusion_expr::{col, lit};
         // int > 1 => c1_max > 1
         let expr = col("c1").gt(lit(15));
         let schema = Schema::new(vec![Field::new("c1", DataType::Int32, false)]);
-        let pruning_predicate = PruningPredicate::try_new(expr, Arc::new(schema))?;
+        let pruning_predicate =
+            PruningPredicate::try_new(expr, Arc::new(schema)).unwrap();
 
         let schema_descr = get_test_schema_descr(vec![("c1", PhysicalType::INT32)]);
         let rgm1 = get_row_group_meta_data(
@@ -1271,26 +1193,22 @@ mod tests {
             &schema_descr,
             vec![ParquetStatistics::int32(Some(11), Some(20), None, 0, false)],
         );
-        let row_group_metadata = vec![rgm1, rgm2];
-        let mut row_group_predicate =
-            build_row_group_predicate(&pruning_predicate, parquet_file_metrics());
-        let row_group_filter = row_group_metadata
-            .iter()
-            .enumerate()
-            .map(|(i, g)| row_group_predicate(g, i))
-            .collect::<Vec<_>>();
-        assert_eq!(row_group_filter, vec![false, true]);
 
-        Ok(())
+        let metrics = parquet_file_metrics();
+        assert_eq!(
+            prune_row_groups(&[rgm1, rgm2], None, Some(pruning_predicate), &metrics),
+            vec![1]
+        );
     }
 
     #[test]
-    fn row_group_pruning_predicate_missing_stats() -> Result<()> {
+    fn row_group_pruning_predicate_missing_stats() {
         use datafusion_expr::{col, lit};
         // int > 1 => c1_max > 1
         let expr = col("c1").gt(lit(15));
         let schema = Schema::new(vec![Field::new("c1", DataType::Int32, false)]);
-        let pruning_predicate = PruningPredicate::try_new(expr, Arc::new(schema))?;
+        let pruning_predicate =
+            PruningPredicate::try_new(expr, Arc::new(schema)).unwrap();
 
         let schema_descr = get_test_schema_descr(vec![("c1", PhysicalType::INT32)]);
         let rgm1 = get_row_group_meta_data(
@@ -1301,23 +1219,17 @@ mod tests {
             &schema_descr,
             vec![ParquetStatistics::int32(Some(11), Some(20), None, 0, false)],
         );
-        let row_group_metadata = vec![rgm1, rgm2];
-        let mut row_group_predicate =
-            build_row_group_predicate(&pruning_predicate, parquet_file_metrics());
-        let row_group_filter = row_group_metadata
-            .iter()
-            .enumerate()
-            .map(|(i, g)| row_group_predicate(g, i))
-            .collect::<Vec<_>>();
+        let metrics = parquet_file_metrics();
         // missing statistics for first row group mean that the result from the predicate expression
         // is null / undefined so the first row group can't be filtered out
-        assert_eq!(row_group_filter, vec![true, true]);
-
-        Ok(())
+        assert_eq!(
+            prune_row_groups(&[rgm1, rgm2], None, Some(pruning_predicate), &metrics),
+            vec![0, 1]
+        );
     }
 
     #[test]
-    fn row_group_pruning_predicate_partial_expr() -> Result<()> {
+    fn row_group_pruning_predicate_partial_expr() {
         use datafusion_expr::{col, lit};
         // test row group predicate with partially supported expression
         // int > 1 and int % 2 => c1_max > 1 and true
@@ -1326,7 +1238,7 @@ mod tests {
             Field::new("c1", DataType::Int32, false),
             Field::new("c2", DataType::Int32, false),
         ]));
-        let pruning_predicate = PruningPredicate::try_new(expr, schema.clone())?;
+        let pruning_predicate = PruningPredicate::try_new(expr, schema.clone()).unwrap();
 
         let schema_descr = get_test_schema_descr(vec![
             ("c1", PhysicalType::INT32),
@@ -1346,32 +1258,27 @@ mod tests {
                 ParquetStatistics::int32(Some(11), Some(20), None, 0, false),
             ],
         );
-        let row_group_metadata = vec![rgm1, rgm2];
-        let mut row_group_predicate =
-            build_row_group_predicate(&pruning_predicate, parquet_file_metrics());
-        let row_group_filter = row_group_metadata
-            .iter()
-            .enumerate()
-            .map(|(i, g)| row_group_predicate(g, i))
-            .collect::<Vec<_>>();
+
+        let metrics = parquet_file_metrics();
+        let groups = &[rgm1, rgm2];
         // the first row group is still filtered out because the predicate expression can be partially evaluated
         // when conditions are joined using AND
-        assert_eq!(row_group_filter, vec![false, true]);
+        assert_eq!(
+            prune_row_groups(groups, None, Some(pruning_predicate), &metrics),
+            vec![1]
+        );
 
         // if conditions in predicate are joined with OR and an unsupported expression is used
         // this bypasses the entire predicate expression and no row groups are filtered out
         let expr = col("c1").gt(lit(15)).or(col("c2").modulus(lit(2)));
-        let pruning_predicate = PruningPredicate::try_new(expr, schema)?;
-        let mut row_group_predicate =
-            build_row_group_predicate(&pruning_predicate, parquet_file_metrics());
-        let row_group_filter = row_group_metadata
-            .iter()
-            .enumerate()
-            .map(|(i, g)| row_group_predicate(g, i))
-            .collect::<Vec<_>>();
-        assert_eq!(row_group_filter, vec![true, true]);
+        let pruning_predicate = PruningPredicate::try_new(expr, schema).unwrap();
 
-        Ok(())
+        // if conditions in predicate are joined with OR and an unsupported expression is used
+        // this bypasses the entire predicate expression and no row groups are filtered out
+        assert_eq!(
+            prune_row_groups(groups, None, Some(pruning_predicate), &metrics),
+            vec![0, 1]
+        );
     }
 
     fn gen_row_group_meta_data_for_pruning_predicate() -> Vec<RowGroupMetaData> {
@@ -1397,7 +1304,7 @@ mod tests {
     }
 
     #[test]
-    fn row_group_pruning_predicate_null_expr() -> Result<()> {
+    fn row_group_pruning_predicate_null_expr() {
         use datafusion_expr::{col, lit};
         // int > 1 and IsNull(bool) => c1_max > 1 and bool_null_count > 0
         let expr = col("c1").gt(lit(15)).and(col("c2").is_null());
@@ -1405,24 +1312,19 @@ mod tests {
             Field::new("c1", DataType::Int32, false),
             Field::new("c2", DataType::Boolean, false),
         ]));
-        let pruning_predicate = PruningPredicate::try_new(expr, schema)?;
-        let row_group_metadata = gen_row_group_meta_data_for_pruning_predicate();
+        let pruning_predicate = PruningPredicate::try_new(expr, schema).unwrap();
+        let groups = gen_row_group_meta_data_for_pruning_predicate();
 
-        let mut row_group_predicate =
-            build_row_group_predicate(&pruning_predicate, parquet_file_metrics());
-        let row_group_filter = row_group_metadata
-            .iter()
-            .enumerate()
-            .map(|(i, g)| row_group_predicate(g, i))
-            .collect::<Vec<_>>();
+        let metrics = parquet_file_metrics();
         // First row group was filtered out because it contains no null value on "c2".
-        assert_eq!(row_group_filter, vec![false, true]);
-
-        Ok(())
+        assert_eq!(
+            prune_row_groups(&groups, None, Some(pruning_predicate), &metrics),
+            vec![1]
+        );
     }
 
     #[test]
-    fn row_group_pruning_predicate_eq_null_expr() -> Result<()> {
+    fn row_group_pruning_predicate_eq_null_expr() {
         use datafusion_expr::{col, lit};
         // test row group predicate with an unknown (Null) expr
         //
@@ -1434,22 +1336,16 @@ mod tests {
             Field::new("c1", DataType::Int32, false),
             Field::new("c2", DataType::Boolean, false),
         ]));
-        let pruning_predicate = PruningPredicate::try_new(expr, schema)?;
-        let row_group_metadata = gen_row_group_meta_data_for_pruning_predicate();
-
-        let mut row_group_predicate =
-            build_row_group_predicate(&pruning_predicate, parquet_file_metrics());
-        let row_group_filter = row_group_metadata
-            .iter()
-            .enumerate()
-            .map(|(i, g)| row_group_predicate(g, i))
-            .collect::<Vec<_>>();
+        let pruning_predicate = PruningPredicate::try_new(expr, schema).unwrap();
+        let groups = gen_row_group_meta_data_for_pruning_predicate();
 
+        let metrics = parquet_file_metrics();
         // bool = NULL always evaluates to NULL (and thus will not
         // pass predicates. Ideally these should both be false
-        assert_eq!(row_group_filter, vec![false, true]);
-
-        Ok(())
+        assert_eq!(
+            prune_row_groups(&groups, None, Some(pruning_predicate), &metrics),
+            vec![1]
+        );
     }
 
     fn get_row_group_meta_data(
