@@ -91,7 +91,7 @@ pub use self::planner::PhysicalPlanner;
 /// Fields are optional and can be inexact because the sources
 /// sometimes provide approximate estimates for performance reasons
 /// and the transformations output are not always predictable.
-#[derive(Debug, Clone, Default, PartialEq)]
+#[derive(Debug, Clone, Default, PartialEq, Eq)]
 pub struct Statistics {
     /// The number of table rows
     pub num_rows: Option<usize>,
@@ -105,7 +105,7 @@ pub struct Statistics {
     pub is_exact: bool,
 }
 /// This table statistics are estimates about column
-#[derive(Clone, Debug, Default, PartialEq)]
+#[derive(Clone, Debug, Default, PartialEq, Eq)]
 pub struct ColumnStatistics {
     /// Number of null values on column
     pub null_count: Option<usize>,
@@ -288,7 +288,7 @@ pub fn with_new_children_if_necessary(
 /// ```
 /// use datafusion::prelude::*;
 /// use datafusion::physical_plan::displayable;
-/// use std::path::is_separator;
+/// use object_store::path::Path;
 ///
 /// #[tokio::main]
 /// async fn main() {
@@ -312,8 +312,8 @@ pub fn with_new_children_if_necessary(
 ///   let plan_string = format!("{}", displayable_plan.indent());
 ///
 ///   let working_directory = std::env::current_dir().unwrap();
-///   let normalized = working_directory.to_string_lossy().replace(is_separator, "/");
-///   let plan_string = plan_string.replace(&normalized, "WORKING_DIR");
+///   let normalized = Path::from_filesystem_path(working_directory).unwrap();
+///   let plan_string = plan_string.replace(normalized.as_ref(), "WORKING_DIR");
 ///
 ///   assert_eq!("ProjectionExec: expr=[a@0 as a]\
 ///              \n  CoalesceBatchesExec: target_batch_size=4096\
