@@ -224,7 +224,7 @@ impl ExecutionPlan for SortPreservingMergeExec {
                     schema,
                     &self.expr,
                     tracking_metrics,
-                    context.session_config().batch_size,
+                    context.session_config().batch_size(),
                 ));
 
                 debug!("Got stream result from SortPreservingMergeStream::new_from_receivers");
@@ -1190,7 +1190,7 @@ mod tests {
             batches.schema(),
             sort.as_slice(),
             tracking_metrics,
-            task_ctx.session_config().batch_size,
+            task_ctx.session_config().batch_size(),
         );
 
         let mut merged = common::collect(Box::pin(merge_stream)).await.unwrap();
