@@ -476,6 +476,205 @@ async fn csv_query_approx_percentile_cont() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn csv_query_cube_avg() -> Result<()> {
+    let ctx = SessionContext::new();
+    register_aggregate_csv_by_sql(&ctx).await;
+
+    let sql = "SELECT c1, c2, AVG(c3) FROM aggregate_test_100 GROUP BY CUBE (c1, c2) ORDER BY c1, c2";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----+----+----------------------------+",
+        "| c1 | c2 | AVG(aggregate_test_100.c3) |",
+        "+----+----+----------------------------+",
+        "| a  | 1  | -17.6                      |",
+        "| a  | 2  | -15.333333333333334        |",
+        "| a  | 3  | -4.5                       |",
+        "| a  | 4  | -32                        |",
+        "| a  | 5  | -32                        |",
+        "| a  |    | -18.333333333333332        |",
+        "| b  | 1  | 31.666666666666668         |",
+        "| b  | 2  | 25.5                       |",
+        "| b  | 3  | -42                        |",
+        "| b  | 4  | -44.6                      |",
+        "| b  | 5  | -0.2                       |",
+        "| b  |    | -5.842105263157895         |",
+        "| c  | 1  | 47.5                       |",
+        "| c  | 2  | -55.57142857142857         |",
+        "| c  | 3  | 47.5                       |",
+        "| c  | 4  | -10.75                     |",
+        "| c  | 5  | 12                         |",
+        "| c  |    | -1.3333333333333333        |",
+        "| d  | 1  | -8.142857142857142         |",
+        "| d  | 2  | 109.33333333333333         |",
+        "| d  | 3  | 41.333333333333336         |",
+        "| d  | 4  | 54                         |",
+        "| d  | 5  | -49.5                      |",
+        "| d  |    | 25.444444444444443         |",
+        "| e  | 1  | 75.66666666666667          |",
+        "| e  | 2  | 37.8                       |",
+        "| e  | 3  | 48                         |",
+        "| e  | 4  | 37.285714285714285         |",
+        "| e  | 5  | -11                        |",
+        "| e  |    | 40.333333333333336         |",
+        "|    | 1  | 16.681818181818183         |",
+        "|    | 2  | 8.363636363636363          |",
+        "|    | 3  | 20.789473684210527         |",
+        "|    | 4  | 1.2608695652173914         |",
+        "|    | 5  | -13.857142857142858        |",
+        "|    |    | 7.81                       |",
+        "+----+----+----------------------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+    Ok(())
+}
+
+#[tokio::test]
+async fn csv_query_rollup_avg() -> Result<()> {
+    let ctx = SessionContext::new();
+    register_aggregate_csv_by_sql(&ctx).await;
+
+    let sql = "SELECT c1, c2, c3, AVG(c4) FROM aggregate_test_100 GROUP BY ROLLUP (c1, c2, c3) ORDER BY c1, c2, c3";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----+----+------+----------------------------+",
+        "| c1 | c2 | c3   | AVG(aggregate_test_100.c4) |",
+        "+----+----+------+----------------------------+",
+        "| a  | 1  | -85  | -15154                     |",
+        "| a  | 1  | -56  | 8692                       |",
+        "| a  | 1  | -25  | 15295                      |",
+        "| a  | 1  | -5   | 12636                      |",
+        "| a  | 1  | 83   | -14704                     |",
+        "| a  | 1  |      | 1353                       |",
+        "| a  | 2  | -48  | -18025                     |",
+        "| a  | 2  | -43  | 13080                      |",
+        "| a  | 2  | 45   | 15673                      |",
+        "| a  | 2  |      | 3576                       |",
+        "| a  | 3  | -72  | -11122                     |",
+        "| a  | 3  | -12  | -9168                      |",
+        "| a  | 3  | 13   | 22338.5                    |",
+        "| a  | 3  | 14   | 28162                      |",
+        "| a  | 3  | 17   | -22796                     |",
+        "| a  | 3  |      | 4958.833333333333          |",
+        "| a  | 4  | -101 | 11640                      |",
+        "| a  | 4  | -54  | -2376                      |",
+        "| a  | 4  | -38  | 20744                      |",
+        "| a  | 4  | 65   | -28462                     |",
+        "| a  | 4  |      | 386.5                      |",
+        "| a  | 5  | -101 | -12484                     |",
+        "| a  | 5  | -31  | -12907                     |",
+        "| a  | 5  | 36   | -16974                     |",
+        "| a  | 5  |      | -14121.666666666666        |",
+        "| a  |    |      | 306.04761904761904         |",
+        "| b  | 1  | 12   | 7652                       |",
+        "| b  | 1  | 29   | -18218                     |",
+        "| b  | 1  | 54   | -18410                     |",
+        "| b  | 1  |      | -9658.666666666666         |",
+        "| b  | 2  | -60  | -21739                     |",
+        "| b  | 2  | 31   | 23127                      |",
+        "| b  | 2  | 63   | 21456                      |",
+        "| b  | 2  | 68   | 15874                      |",
+        "| b  | 2  |      | 9679.5                     |",
+        "| b  | 3  | -101 | -13217                     |",
+        "| b  | 3  | 17   | 14457                      |",
+        "| b  | 3  |      | 620                        |",
+        "| b  | 4  | -117 | 19316                      |",
+        "| b  | 4  | -111 | -1967                      |",
+        "| b  | 4  | -59  | 25286                      |",
+        "| b  | 4  | 17   | -28070                     |",
+        "| b  | 4  | 47   | 20690                      |",
+        "| b  | 4  |      | 7051                       |",
+        "| b  | 5  | -82  | 22080                      |",
+        "| b  | 5  | -44  | 15788                      |",
+        "| b  | 5  | -5   | 24896                      |",
+        "| b  | 5  | 62   | 16337                      |",
+        "| b  | 5  | 68   | 21576                      |",
+        "| b  | 5  |      | 20135.4                    |",
+        "| b  |    |      | 7732.315789473684          |",
+        "| c  | 1  | -24  | -24085                     |",
+        "| c  | 1  | 41   | -4667                      |",
+        "| c  | 1  | 70   | 27752                      |",
+        "| c  | 1  | 103  | -22186                     |",
+        "| c  | 1  |      | -5796.5                    |",
+        "| c  | 2  | -117 | -30187                     |",
+        "| c  | 2  | -107 | -2904                      |",
+        "| c  | 2  | -106 | -1114                      |",
+        "| c  | 2  | -60  | -16312                     |",
+        "| c  | 2  | -29  | 25305                      |",
+        "| c  | 2  | 1    | 18109                      |",
+        "| c  | 2  | 29   | -3855                      |",
+        "| c  | 2  |      | -1565.4285714285713        |",
+        "| c  | 3  | -2   | -18655                     |",
+        "| c  | 3  | 22   | 13741                      |",
+        "| c  | 3  | 73   | -9565                      |",
+        "| c  | 3  | 97   | 29106                      |",
+        "| c  | 3  |      | 3656.75                    |",
+        "| c  | 4  | -90  | -2935                      |",
+        "| c  | 4  | -79  | 5281                       |",
+        "| c  | 4  | 3    | -30508                     |",
+        "| c  | 4  | 123  | 16620                      |",
+        "| c  | 4  |      | -2885.5                    |",
+        "| c  | 5  | -94  | -15880                     |",
+        "| c  | 5  | 118  | 19208                      |",
+        "| c  | 5  |      | 1664                       |",
+        "| c  |    |      | -1320.5238095238096        |",
+        "| d  | 1  | -99  | 5613                       |",
+        "| d  | 1  | -98  | 13630                      |",
+        "| d  | 1  | -72  | 25590                      |",
+        "| d  | 1  | -8   | 27138                      |",
+        "| d  | 1  | 38   | 18384                      |",
+        "| d  | 1  | 57   | 28781                      |",
+        "| d  | 1  | 125  | 31106                      |",
+        "| d  | 1  |      | 21463.14285714286          |",
+        "| d  | 2  | 93   | -12642                     |",
+        "| d  | 2  | 113  | 3917                       |",
+        "| d  | 2  | 122  | 10130                      |",
+        "| d  | 2  |      | 468.3333333333333          |",
+        "| d  | 3  | -76  | 8809                       |",
+        "| d  | 3  | 77   | 15091                      |",
+        "| d  | 3  | 123  | 29533                      |",
+        "| d  | 3  |      | 17811                      |",
+        "| d  | 4  | 5    | -7688                      |",
+        "| d  | 4  | 55   | -1471                      |",
+        "| d  | 4  | 102  | -24558                     |",
+        "| d  | 4  |      | -11239                     |",
+        "| d  | 5  | -59  | 2045                       |",
+        "| d  | 5  | -40  | 22614                      |",
+        "| d  | 5  |      | 12329.5                    |",
+        "| d  |    |      | 10890.111111111111         |",
+        "| e  | 1  | 36   | -21481                     |",
+        "| e  | 1  | 71   | -5479                      |",
+        "| e  | 1  | 120  | 10837                      |",
+        "| e  | 1  |      | -5374.333333333333         |",
+        "| e  | 2  | -61  | -2888                      |",
+        "| e  | 2  | 49   | 24495                      |",
+        "| e  | 2  | 52   | 5666                       |",
+        "| e  | 2  | 97   | 18167                      |",
+        "| e  | 2  |      | 10221.2                    |",
+        "| e  | 3  | -95  | 13611                      |",
+        "| e  | 3  | 71   | 194                        |",
+        "| e  | 3  | 104  | -25136                     |",
+        "| e  | 3  | 112  | -6823                      |",
+        "| e  | 3  |      | -4538.5                    |",
+        "| e  | 4  | -56  | -31500                     |",
+        "| e  | 4  | -53  | 13788                      |",
+        "| e  | 4  | 30   | -16110                     |",
+        "| e  | 4  | 73   | -22501                     |",
+        "| e  | 4  | 74   | -12612                     |",
+        "| e  | 4  | 96   | -30336                     |",
+        "| e  | 4  | 97   | -13181                     |",
+        "| e  | 4  |      | -16064.57142857143         |",
+        "| e  | 5  | -86  | 32514                      |",
+        "| e  | 5  | 64   | -26526                     |",
+        "| e  | 5  |      | 2994                       |",
+        "| e  |    |      | -4268.333333333333         |",
+        "|    |    |      | 2319.97                    |",
+        "+----+----+------+----------------------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+    Ok(())
+}
+
 #[tokio::test]
 async fn csv_query_approx_percentile_cont_with_weight() -> Result<()> {
     let ctx = SessionContext::new();
@@ -583,6 +782,200 @@ async fn csv_query_sum_crossjoin() {
     assert_batches_eq!(expected, &actual);
 }
 
+#[tokio::test]
+async fn csv_query_cube_sum_crossjoin() {
+    let ctx = SessionContext::new();
+    register_aggregate_csv_by_sql(&ctx).await;
+    let sql = "SELECT a.c1, b.c1, SUM(a.c2) FROM aggregate_test_100 as a CROSS JOIN aggregate_test_100 as b GROUP BY CUBE (a.c1, b.c1) ORDER BY a.c1, b.c1";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----+----+-----------+",
+        "| c1 | c1 | SUM(a.c2) |",
+        "+----+----+-----------+",
+        "| a  | a  | 1260      |",
+        "| a  | b  | 1140      |",
+        "| a  | c  | 1260      |",
+        "| a  | d  | 1080      |",
+        "| a  | e  | 1260      |",
+        "| a  |    | 6000      |",
+        "| b  | a  | 1302      |",
+        "| b  | b  | 1178      |",
+        "| b  | c  | 1302      |",
+        "| b  | d  | 1116      |",
+        "| b  | e  | 1302      |",
+        "| b  |    | 6200      |",
+        "| c  | a  | 1176      |",
+        "| c  | b  | 1064      |",
+        "| c  | c  | 1176      |",
+        "| c  | d  | 1008      |",
+        "| c  | e  | 1176      |",
+        "| c  |    | 5600      |",
+        "| d  | a  | 924       |",
+        "| d  | b  | 836       |",
+        "| d  | c  | 924       |",
+        "| d  | d  | 792       |",
+        "| d  | e  | 924       |",
+        "| d  |    | 4400      |",
+        "| e  | a  | 1323      |",
+        "| e  | b  | 1197      |",
+        "| e  | c  | 1323      |",
+        "| e  | d  | 1134      |",
+        "| e  | e  | 1323      |",
+        "| e  |    | 6300      |",
+        "|    | a  | 5985      |",
+        "|    | b  | 5415      |",
+        "|    | c  | 5985      |",
+        "|    | d  | 5130      |",
+        "|    | e  | 5985      |",
+        "|    |    | 28500     |",
+        "+----+----+-----------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+}
+
+#[tokio::test]
+async fn csv_query_cube_distinct_count() {
+    let ctx = SessionContext::new();
+    register_aggregate_csv_by_sql(&ctx).await;
+    let sql = "SELECT c1, c2, COUNT(DISTINCT c3) FROM aggregate_test_100 GROUP BY CUBE (c1,c2) ORDER BY c1,c2";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----+----+---------------------------------------+",
+        "| c1 | c2 | COUNT(DISTINCT aggregate_test_100.c3) |",
+        "+----+----+---------------------------------------+",
+        "| a  | 1  | 5                                     |",
+        "| a  | 2  | 3                                     |",
+        "| a  | 3  | 5                                     |",
+        "| a  | 4  | 4                                     |",
+        "| a  | 5  | 3                                     |",
+        "| a  |    | 19                                    |",
+        "| b  | 1  | 3                                     |",
+        "| b  | 2  | 4                                     |",
+        "| b  | 3  | 2                                     |",
+        "| b  | 4  | 5                                     |",
+        "| b  | 5  | 5                                     |",
+        "| b  |    | 17                                    |",
+        "| c  | 1  | 4                                     |",
+        "| c  | 2  | 7                                     |",
+        "| c  | 3  | 4                                     |",
+        "| c  | 4  | 4                                     |",
+        "| c  | 5  | 2                                     |",
+        "| c  |    | 21                                    |",
+        "| d  | 1  | 7                                     |",
+        "| d  | 2  | 3                                     |",
+        "| d  | 3  | 3                                     |",
+        "| d  | 4  | 3                                     |",
+        "| d  | 5  | 2                                     |",
+        "| d  |    | 18                                    |",
+        "| e  | 1  | 3                                     |",
+        "| e  | 2  | 4                                     |",
+        "| e  | 3  | 4                                     |",
+        "| e  | 4  | 7                                     |",
+        "| e  | 5  | 2                                     |",
+        "| e  |    | 18                                    |",
+        "|    | 1  | 22                                    |",
+        "|    | 2  | 20                                    |",
+        "|    | 3  | 17                                    |",
+        "|    | 4  | 23                                    |",
+        "|    | 5  | 14                                    |",
+        "|    |    | 80                                    |",
+        "+----+----+---------------------------------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+}
+
+#[tokio::test]
+async fn csv_query_rollup_distinct_count() {
+    let ctx = SessionContext::new();
+    register_aggregate_csv_by_sql(&ctx).await;
+    let sql = "SELECT c1, c2, COUNT(DISTINCT c3) FROM aggregate_test_100 GROUP BY ROLLUP (c1,c2) ORDER BY c1,c2";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----+----+---------------------------------------+",
+        "| c1 | c2 | COUNT(DISTINCT aggregate_test_100.c3) |",
+        "+----+----+---------------------------------------+",
+        "| a  | 1  | 5                                     |",
+        "| a  | 2  | 3                                     |",
+        "| a  | 3  | 5                                     |",
+        "| a  | 4  | 4                                     |",
+        "| a  | 5  | 3                                     |",
+        "| a  |    | 19                                    |",
+        "| b  | 1  | 3                                     |",
+        "| b  | 2  | 4                                     |",
+        "| b  | 3  | 2                                     |",
+        "| b  | 4  | 5                                     |",
+        "| b  | 5  | 5                                     |",
+        "| b  |    | 17                                    |",
+        "| c  | 1  | 4                                     |",
+        "| c  | 2  | 7                                     |",
+        "| c  | 3  | 4                                     |",
+        "| c  | 4  | 4                                     |",
+        "| c  | 5  | 2                                     |",
+        "| c  |    | 21                                    |",
+        "| d  | 1  | 7                                     |",
+        "| d  | 2  | 3                                     |",
+        "| d  | 3  | 3                                     |",
+        "| d  | 4  | 3                                     |",
+        "| d  | 5  | 2                                     |",
+        "| d  |    | 18                                    |",
+        "| e  | 1  | 3                                     |",
+        "| e  | 2  | 4                                     |",
+        "| e  | 3  | 4                                     |",
+        "| e  | 4  | 7                                     |",
+        "| e  | 5  | 2                                     |",
+        "| e  |    | 18                                    |",
+        "|    |    | 80                                    |",
+        "+----+----+---------------------------------------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+}
+
+#[tokio::test]
+async fn csv_query_rollup_sum_crossjoin() {
+    let ctx = SessionContext::new();
+    register_aggregate_csv_by_sql(&ctx).await;
+    let sql = "SELECT a.c1, b.c1, SUM(a.c2) FROM aggregate_test_100 as a CROSS JOIN aggregate_test_100 as b GROUP BY ROLLUP (a.c1, b.c1) ORDER BY a.c1, b.c1";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----+----+-----------+",
+        "| c1 | c1 | SUM(a.c2) |",
+        "+----+----+-----------+",
+        "| a  | a  | 1260      |",
+        "| a  | b  | 1140      |",
+        "| a  | c  | 1260      |",
+        "| a  | d  | 1080      |",
+        "| a  | e  | 1260      |",
+        "| a  |    | 6000      |",
+        "| b  | a  | 1302      |",
+        "| b  | b  | 1178      |",
+        "| b  | c  | 1302      |",
+        "| b  | d  | 1116      |",
+        "| b  | e  | 1302      |",
+        "| b  |    | 6200      |",
+        "| c  | a  | 1176      |",
+        "| c  | b  | 1064      |",
+        "| c  | c  | 1176      |",
+        "| c  | d  | 1008      |",
+        "| c  | e  | 1176      |",
+        "| c  |    | 5600      |",
+        "| d  | a  | 924       |",
+        "| d  | b  | 836       |",
+        "| d  | c  | 924       |",
+        "| d  | d  | 792       |",
+        "| d  | e  | 924       |",
+        "| d  |    | 4400      |",
+        "| e  | a  | 1323      |",
+        "| e  | b  | 1197      |",
+        "| e  | c  | 1323      |",
+        "| e  | d  | 1134      |",
+        "| e  | e  | 1323      |",
+        "| e  |    | 6300      |",
+        "|    |    | 28500     |",
+        "+----+----+-----------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+}
+
 #[tokio::test]
 async fn query_count_without_from() -> Result<()> {
     let ctx = SessionContext::new();
@@ -675,6 +1068,59 @@ async fn csv_query_array_agg_with_overflow() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn csv_query_array_cube_agg_with_overflow() -> Result<()> {
+    let ctx = SessionContext::new();
+    register_aggregate_csv(&ctx).await?;
+    let sql =
+        "select c1, c2, sum(c3) sum_c3, avg(c3) avg_c3, max(c3) max_c3, min(c3) min_c3, count(c3) count_c3 from aggregate_test_100 group by CUBE (c1,c2) order by c1, c2";
+    let actual = execute_to_batches(&ctx, sql).await;
+    let expected = vec![
+        "+----+----+--------+---------------------+--------+--------+----------+",
+        "| c1 | c2 | sum_c3 | avg_c3              | max_c3 | min_c3 | count_c3 |",
+        "+----+----+--------+---------------------+--------+--------+----------+",
+        "| a  | 1  | -88    | -17.6               | 83     | -85    | 5        |",
+        "| a  | 2  | -46    | -15.333333333333334 | 45     | -48    | 3        |",
+        "| a  | 3  | -27    | -4.5                | 17     | -72    | 6        |",
+        "| a  | 4  | -128   | -32                 | 65     | -101   | 4        |",
+        "| a  | 5  | -96    | -32                 | 36     | -101   | 3        |",
+        "| a  |    | -385   | -18.333333333333332 | 83     | -101   | 21       |",
+        "| b  | 1  | 95     | 31.666666666666668  | 54     | 12     | 3        |",
+        "| b  | 2  | 102    | 25.5                | 68     | -60    | 4        |",
+        "| b  | 3  | -84    | -42                 | 17     | -101   | 2        |",
+        "| b  | 4  | -223   | -44.6               | 47     | -117   | 5        |",
+        "| b  | 5  | -1     | -0.2                | 68     | -82    | 5        |",
+        "| b  |    | -111   | -5.842105263157895  | 68     | -117   | 19       |",
+        "| c  | 1  | 190    | 47.5                | 103    | -24    | 4        |",
+        "| c  | 2  | -389   | -55.57142857142857  | 29     | -117   | 7        |",
+        "| c  | 3  | 190    | 47.5                | 97     | -2     | 4        |",
+        "| c  | 4  | -43    | -10.75              | 123    | -90    | 4        |",
+        "| c  | 5  | 24     | 12                  | 118    | -94    | 2        |",
+        "| c  |    | -28    | -1.3333333333333333 | 123    | -117   | 21       |",
+        "| d  | 1  | -57    | -8.142857142857142  | 125    | -99    | 7        |",
+        "| d  | 2  | 328    | 109.33333333333333  | 122    | 93     | 3        |",
+        "| d  | 3  | 124    | 41.333333333333336  | 123    | -76    | 3        |",
+        "| d  | 4  | 162    | 54                  | 102    | 5      | 3        |",
+        "| d  | 5  | -99    | -49.5               | -40    | -59    | 2        |",
+        "| d  |    | 458    | 25.444444444444443  | 125    | -99    | 18       |",
+        "| e  | 1  | 227    | 75.66666666666667   | 120    | 36     | 3        |",
+        "| e  | 2  | 189    | 37.8                | 97     | -61    | 5        |",
+        "| e  | 3  | 192    | 48                  | 112    | -95    | 4        |",
+        "| e  | 4  | 261    | 37.285714285714285  | 97     | -56    | 7        |",
+        "| e  | 5  | -22    | -11                 | 64     | -86    | 2        |",
+        "| e  |    | 847    | 40.333333333333336  | 120    | -95    | 21       |",
+        "|    | 1  | 367    | 16.681818181818183  | 125    | -99    | 22       |",
+        "|    | 2  | 184    | 8.363636363636363   | 122    | -117   | 22       |",
+        "|    | 3  | 395    | 20.789473684210527  | 123    | -101   | 19       |",
+        "|    | 4  | 29     | 1.2608695652173914  | 123    | -117   | 23       |",
+        "|    | 5  | -194   | -13.857142857142858 | 118    | -101   | 14       |",
+        "|    |    | 781    | 7.81                | 125    | -117   | 100      |",
+        "+----+----+--------+---------------------+--------+--------+----------+",
+    ];
+    assert_batches_eq!(expected, &actual);
+    Ok(())
+}
+
 #[tokio::test]
 async fn csv_query_array_agg_distinct() -> Result<()> {
     let ctx = SessionContext::new();
@@ -1223,6 +1669,79 @@ async fn count_aggregated() -> Result<()> {
     Ok(())
 }
 
+#[tokio::test]
+async fn count_aggregated_cube() -> Result<()> {
+    let results = execute_with_partition(
+        "SELECT c1, c2, COUNT(c3) FROM test GROUP BY CUBE (c1, c2) ORDER BY c1, c2",
+        4,
+    )
+    .await?;
+
+    let expected = vec![
+        "+----+----+----------------+",
+        "| c1 | c2 | COUNT(test.c3) |",
+        "+----+----+----------------+",
+        "|    |    | 40             |",
+        "|    | 1  | 4              |",
+        "|    | 10 | 4              |",
+        "|    | 2  | 4              |",
+        "|    | 3  | 4              |",
+        "|    | 4  | 4              |",
+        "|    | 5  | 4              |",
+        "|    | 6  | 4              |",
+        "|    | 7  | 4              |",
+        "|    | 8  | 4              |",
+        "|    | 9  | 4              |",
+        "| 0  |    | 10             |",
+        "| 0  | 1  | 1              |",
+        "| 0  | 10 | 1              |",
+        "| 0  | 2  | 1              |",
+        "| 0  | 3  | 1              |",
+        "| 0  | 4  | 1              |",
+        "| 0  | 5  | 1              |",
+        "| 0  | 6  | 1              |",
+        "| 0  | 7  | 1              |",
+        "| 0  | 8  | 1              |",
+        "| 0  | 9  | 1              |",
+        "| 1  |    | 10             |",
+        "| 1  | 1  | 1              |",
+        "| 1  | 10 | 1              |",
+        "| 1  | 2  | 1              |",
+        "| 1  | 3  | 1              |",
+        "| 1  | 4  | 1              |",
+        "| 1  | 5  | 1              |",
+        "| 1  | 6  | 1              |",
+        "| 1  | 7  | 1              |",
+        "| 1  | 8  | 1              |",
+        "| 1  | 9  | 1              |",
+        "| 2  |    | 10             |",
+        "| 2  | 1  | 1              |",
+        "| 2  | 10 | 1              |",
+        "| 2  | 2  | 1              |",
+        "| 2  | 3  | 1              |",
+        "| 2  | 4  | 1              |",
+        "| 2  | 5  | 1              |",
+        "| 2  | 6  | 1              |",
+        "| 2  | 7  | 1              |",
+        "| 2  | 8  | 1              |",
+        "| 2  | 9  | 1              |",
+        "| 3  |    | 10             |",
+        "| 3  | 1  | 1              |",
+        "| 3  | 10 | 1              |",
+        "| 3  | 2  | 1              |",
+        "| 3  | 3  | 1              |",
+        "| 3  | 4  | 1              |",
+        "| 3  | 5  | 1              |",
+        "| 3  | 6  | 1              |",
+        "| 3  | 7  | 1              |",
+        "| 3  | 8  | 1              |",
+        "| 3  | 9  | 1              |",
+        "+----+----+----------------+",
+    ];
+    assert_batches_sorted_eq!(expected, &results);
+    Ok(())
+}
+
 #[tokio::test]
 async fn simple_avg() -> Result<()> {
     let schema = Schema::new(vec![Field::new("a", DataType::Int32, false)]);
