@@ -46,7 +46,7 @@
 //! | logical_plan | Limit: 3                               |
 //! |              |   Sort: #revenue DESC NULLS FIRST      |
 //! |              |     Projection: #customer_id, #revenue |
-//! |              |       TableScan: sales projection=None |
+//! |              |       TableScan: sales |
 //! +--------------+----------------------------------------+
 //! ```
 //!
@@ -219,7 +219,7 @@ async fn topk_plan() -> Result<()> {
     let mut expected = vec![
         "| logical_plan after topk                               | TopK: k=3                                                                     |",
         "|                                                       |   Projection: #sales.customer_id, #sales.revenue                              |",
-        "|                                                       |     TableScan: sales projection=Some([customer_id, revenue])                                  |",
+        "|                                                       |     TableScan: sales projection=[customer_id,revenue]                                  |",
     ].join("\n");
 
     let explain_query = format!("EXPLAIN VERBOSE {}", QUERY);
@@ -367,7 +367,7 @@ impl UserDefinedLogicalNode for TopKPlanNode {
         &self,
         exprs: &[Expr],
         inputs: &[LogicalPlan],
-    ) -> Arc<dyn UserDefinedLogicalNode + Send + Sync> {
+    ) -> Arc<dyn UserDefinedLogicalNode> {
         assert_eq!(inputs.len(), 1, "input size inconsistent");
         assert_eq!(exprs.len(), 1, "expression size inconsistent");
         Arc::new(TopKPlanNode {
@@ -381,9 +381,10 @@ impl UserDefinedLogicalNode for TopKPlanNode {
 /// Physical planner for TopK nodes
 struct TopKPlanner {}
 
+#[async_trait]
 impl ExtensionPlanner for TopKPlanner {
     /// Create a physical plan for an extension node
-    fn plan_extension(
+    async fn plan_extension(
         &self,
         _planner: &dyn PhysicalPlanner,
         node: &dyn UserDefinedLogicalNode,
