@@ -18,11 +18,13 @@
 //! This module provides a builder for creating LogicalPlans
 
 use crate::expr_rewriter::{normalize_col, normalize_cols, rewrite_sort_cols_by_aggs};
-use crate::utils::{columnize_expr, exprlist_to_fields, from_plan};
+use crate::utils::{
+    columnize_expr, exprlist_to_fields, from_plan, grouping_set_to_exprlist,
+};
 use crate::{and, binary_expr, Operator};
 use crate::{
     logical_plan::{
-        Aggregate, Analyze, CrossJoin, EmptyRelation, Explain, Filter, Join,
+        Aggregate, Analyze, CrossJoin, Distinct, EmptyRelation, Explain, Filter, Join,
         JoinConstraint, JoinType, Limit, LogicalPlan, Partitioning, PlanType, Projection,
         Repartition, Sort, SubqueryAlias, TableScan, ToStringifiedPlan, Union, Values,
         Window,
@@ -40,7 +42,6 @@ use datafusion_common::{
 };
 use std::any::Any;
 use std::convert::TryFrom;
-use std::iter;
 use std::{
     collections::{HashMap, HashSet},
     sync::Arc,
@@ -435,16 +436,27 @@ impl LogicalPlanBuilder {
 
     /// Apply a union, removing duplicate rows
     pub fn union_distinct(&self, plan: LogicalPlan) -> Result<Self> {
-        self.union(plan)?.distinct()
+        // unwrap top-level Distincts, to avoid duplication
+        let left_plan = self.plan.clone();
+        let left_plan: LogicalPlan = match left_plan {
+            LogicalPlan::Distinct(Distinct { input }) => (*input).clone(),
+            _ => left_plan,
+        };
+        let right_plan: LogicalPlan = match plan {
+            LogicalPlan::Distinct(Distinct { input }) => (*input).clone(),
+            _ => plan,
+        };
+
+        Ok(Self::from(LogicalPlan::Distinct(Distinct {
+            input: Arc::new(union_with_alias(left_plan, right_plan, None)?),
+        })))
     }
 
     /// Apply deduplication: Only distinct (different) values are returned)
     pub fn distinct(&self) -> Result<Self> {
-        let projection_expr = expand_wildcard(self.plan.schema(), &self.plan)?;
-        let plan = LogicalPlanBuilder::from(self.plan.clone())
-            .aggregate(projection_expr, iter::empty::<Expr>())?
-            .build()?;
-        Self::from(plan).project(vec![Expr::Wildcard])
+        Ok(Self::from(LogicalPlan::Distinct(Distinct {
+            input: Arc::new(self.plan.clone()),
+        })))
     }
 
     /// Apply a join with on constraint.
@@ -694,7 +706,10 @@ impl LogicalPlanBuilder {
     ) -> Result<Self> {
         let group_expr = normalize_cols(group_expr, &self.plan)?;
         let aggr_expr = normalize_cols(aggr_expr, &self.plan)?;
-        let all_expr = group_expr.iter().chain(aggr_expr.iter());
+
+        let grouping_expr: Vec<Expr> = grouping_set_to_exprlist(group_expr.as_slice())?;
+
+        let all_expr = grouping_expr.iter().chain(aggr_expr.iter());
         validate_unique_names("Aggregations", all_expr.clone(), self.plan.schema())?;
         let aggr_schema = DFSchema::new_with_metadata(
             exprlist_to_fields(all_expr, &self.plan)?,
@@ -866,7 +881,7 @@ pub fn project_with_column_index_alias(
         .map(|(i, e)| match e {
             ignore_alias @ Expr::Alias { .. } => ignore_alias,
             ignore_col @ Expr::Column { .. } => ignore_col,
-            x => x.alias(format!("column{}", i).as_str()),
+            x => x.alias(schema.field(i).name()),
         })
         .collect::<Vec<_>>();
     Ok(LogicalPlan::Projection(Projection {
@@ -888,7 +903,7 @@ pub fn union_with_alias(
         .into_iter()
         .flat_map(|p| match p {
             LogicalPlan::Union(Union { inputs, .. }) => inputs,
-            x => vec![x],
+            x => vec![Arc::new(x)],
         });
 
     inputs_iter
@@ -901,16 +916,22 @@ pub fn union_with_alias(
         })?;
 
     let inputs = inputs_iter
-        .map(|p| match p {
+        .map(|p| match p.as_ref() {
             LogicalPlan::Projection(Projection {
                 expr, input, alias, ..
-            }) => {
-                project_with_column_index_alias(expr, input, union_schema.clone(), alias)
-                    .unwrap()
-            }
-            x => x,
+            }) => Arc::new(
+                project_with_column_index_alias(
+                    expr.to_vec(),
+                    input.clone(),
+                    union_schema.clone(),
+                    alias.clone(),
+                )
+                .unwrap(),
+            ),
+            x => Arc::new(x.clone()),
         })
         .collect::<Vec<_>>();
+
     if inputs.is_empty() {
         return Err(DataFusionError::Plan("Empty UNION".to_string()));
     }
@@ -1027,7 +1048,7 @@ mod tests {
 
         let expected = "Projection: #employee_csv.id\
         \n  Filter: #employee_csv.state = Utf8(\"CO\")\
-        \n    TableScan: employee_csv projection=Some([id, state])";
+        \n    TableScan: employee_csv projection=[id, state]";
 
         assert_eq!(expected, format!("{:?}", plan));
 
@@ -1058,9 +1079,9 @@ mod tests {
                 .build()?;
 
         let expected = "Limit: skip=2, fetch=10\
-        \n  Projection: #employee_csv.state, #total_salary\
-        \n    Aggregate: groupBy=[[#employee_csv.state]], aggr=[[SUM(#employee_csv.salary) AS total_salary]]\
-        \n      TableScan: employee_csv projection=Some([state, salary])";
+                \n  Projection: #employee_csv.state, #total_salary\
+                \n    Aggregate: groupBy=[[#employee_csv.state]], aggr=[[SUM(#employee_csv.salary) AS total_salary]]\
+                \n      TableScan: employee_csv projection=[state, salary]";
 
         assert_eq!(expected, format!("{:?}", plan));
 
@@ -1086,7 +1107,7 @@ mod tests {
                 .build()?;
 
         let expected = "Sort: #employee_csv.state ASC NULLS FIRST, #employee_csv.salary DESC NULLS LAST\
-        \n  TableScan: employee_csv projection=Some([state, salary])";
+        \n  TableScan: employee_csv projection=[state, salary]";
 
         assert_eq!(expected, format!("{:?}", plan));
 
@@ -1105,8 +1126,8 @@ mod tests {
         // id column should only show up once in projection
         let expected = "Projection: #t1.id, #t1.first_name, #t1.last_name, #t1.state, #t1.salary, #t2.first_name, #t2.last_name, #t2.state, #t2.salary\
         \n  Inner Join: Using #t1.id = #t2.id\
-        \n    TableScan: t1 projection=None\
-        \n    TableScan: t2 projection=None";
+        \n    TableScan: t1\
+        \n    TableScan: t2";
 
         assert_eq!(expected, format!("{:?}", plan));
 
@@ -1126,10 +1147,55 @@ mod tests {
 
         // output has only one union
         let expected = "Union\
-        \n  TableScan: employee_csv projection=Some([state, salary])\
-        \n  TableScan: employee_csv projection=Some([state, salary])\
-        \n  TableScan: employee_csv projection=Some([state, salary])\
-        \n  TableScan: employee_csv projection=Some([state, salary])";
+        \n  TableScan: employee_csv projection=[state, salary]\
+        \n  TableScan: employee_csv projection=[state, salary]\
+        \n  TableScan: employee_csv projection=[state, salary]\
+        \n  TableScan: employee_csv projection=[state, salary]";
+
+        assert_eq!(expected, format!("{:?}", plan));
+
+        Ok(())
+    }
+
+    #[test]
+    fn plan_builder_union_distinct_combined_single_union() -> Result<()> {
+        let plan =
+            table_scan(Some("employee_csv"), &employee_schema(), Some(vec![3, 4]))?;
+
+        let plan = plan
+            .union_distinct(plan.build()?)?
+            .union_distinct(plan.build()?)?
+            .union_distinct(plan.build()?)?
+            .build()?;
+
+        // output has only one union
+        let expected = "\
+        Distinct:\
+        \n  Union\
+        \n    TableScan: employee_csv projection=[state, salary]\
+        \n    TableScan: employee_csv projection=[state, salary]\
+        \n    TableScan: employee_csv projection=[state, salary]\
+        \n    TableScan: employee_csv projection=[state, salary]";
+
+        assert_eq!(expected, format!("{:?}", plan));
+
+        Ok(())
+    }
+
+    #[test]
+    fn plan_builder_simple_distinct() -> Result<()> {
+        let plan =
+            table_scan(Some("employee_csv"), &employee_schema(), Some(vec![0, 3]))?
+                .filter(col("state").eq(lit("CO")))?
+                .project(vec![col("id")])?
+                .distinct()?
+                .build()?;
+
+        let expected = "\
+        Distinct:\
+        \n  Projection: #employee_csv.id\
+        \n    Filter: #employee_csv.state = Utf8(\"CO\")\
+        \n      TableScan: employee_csv projection=[id, state]";
 
         assert_eq!(expected, format!("{:?}", plan));
 
@@ -1154,8 +1220,8 @@ mod tests {
         let expected = "Filter: EXISTS (\
             Subquery: Filter: #foo.a = #bar.a\
             \n  Projection: #foo.a\
-            \n    TableScan: foo projection=None)\
-        \n  Projection: #bar.a\n    TableScan: bar projection=None";
+            \n    TableScan: foo)\
+        \n  Projection: #bar.a\n    TableScan: bar";
         assert_eq!(expected, format!("{:?}", outer_query));
 
         Ok(())
@@ -1179,9 +1245,9 @@ mod tests {
 
         let expected = "Filter: #bar.a IN (Subquery: Filter: #foo.a = #bar.a\
         \n  Projection: #foo.a\
-        \n    TableScan: foo projection=None)\
+        \n    TableScan: foo)\
         \n  Projection: #bar.a\
-        \n    TableScan: bar projection=None";
+        \n    TableScan: bar";
         assert_eq!(expected, format!("{:?}", outer_query));
 
         Ok(())
@@ -1204,8 +1270,8 @@ mod tests {
 
         let expected = "Projection: (Subquery: Filter: #foo.a = #bar.a\
                 \n  Projection: #foo.b\
-                \n    TableScan: foo projection=None)\
-            \n  TableScan: bar projection=None";
+                \n    TableScan: foo)\
+            \n  TableScan: bar";
         assert_eq!(expected, format!("{:?}", outer_query));
 
         Ok(())
