@@ -30,6 +30,7 @@ use datafusion_expr::{
     Expr, ExprSchemable,
 };
 use std::collections::{HashMap, HashSet};
+use std::fmt::Write;
 use std::sync::Arc;
 
 /// A map from expression's identifier to tuple including
@@ -230,6 +231,7 @@ fn optimize(
         | LogicalPlan::CreateCatalogSchema(_)
         | LogicalPlan::CreateCatalog(_)
         | LogicalPlan::DropTable(_)
+        | LogicalPlan::Distinct(_)
         | LogicalPlan::Extension { .. } => {
             // apply the optimization to all inputs of the plan
             let expr = plan.expressions();
@@ -261,10 +263,6 @@ fn to_arrays(
 }
 
 /// Build the "intermediate" projection plan that evaluates the extracted common expressions.
-///
-/// This projection plan will merge all fields in the `input.schema()` into its own schema.
-/// Redundant project fields are expected to be removed in other optimize phase (like
-/// `projection_push_down`).
 fn build_project_plan(
     input: LogicalPlan,
     affected_id: HashSet<Identifier>,
@@ -272,18 +270,24 @@ fn build_project_plan(
 ) -> Result<LogicalPlan> {
     let mut project_exprs = vec![];
     let mut fields = vec![];
+    let mut fields_set = HashSet::new();
 
     for id in affected_id {
         let (expr, _, data_type) = expr_set.get(&id).unwrap();
         // todo: check `nullable`
-        fields.push(DFField::new(None, &id, data_type.clone(), true));
+        let field = DFField::new(None, &id, data_type.clone(), true);
+        fields_set.insert(field.name().to_owned());
+        fields.push(field);
         project_exprs.push(expr.clone().alias(&id));
     }
 
-    fields.extend_from_slice(input.schema().fields());
-    input.schema().fields().iter().for_each(|field| {
-        project_exprs.push(Expr::Column(field.qualified_column()));
-    });
+    for field in input.schema().fields() {
+        if !fields_set.contains(field.name()) {
+            fields_set.insert(field.name().to_owned());
+            fields.push(field.clone());
+            project_exprs.push(Expr::Column(field.qualified_column()));
+        }
+    }
 
     let mut schema = DFSchema::new_with_metadata(fields, HashMap::new())?;
     schema.merge(input.schema());
@@ -424,17 +428,17 @@ impl ExprIdentifierVisitor<'_> {
             }
             Expr::Cast { data_type, .. } => {
                 desc.push_str("Cast-");
-                desc.push_str(&format!("{:?}", data_type));
+                let _ = write!(desc, "{:?}", data_type);
             }
             Expr::TryCast { data_type, .. } => {
                 desc.push_str("TryCast-");
-                desc.push_str(&format!("{:?}", data_type));
+                let _ = write!(desc, "{:?}", data_type);
             }
             Expr::Sort {
                 asc, nulls_first, ..
             } => {
                 desc.push_str("Sort-");
-                desc.push_str(&format!("{}{}", asc, nulls_first));
+                let _ = write!(desc, "{}{}", asc, nulls_first);
             }
             Expr::ScalarFunction { fun, .. } => {
                 desc.push_str("ScalarFunction-");
@@ -449,7 +453,7 @@ impl ExprIdentifierVisitor<'_> {
             } => {
                 desc.push_str("WindowFunction-");
                 desc.push_str(&fun.to_string());
-                desc.push_str(&format!("{:?}", window_frame));
+                let _ = write!(desc, "{:?}", window_frame);
             }
             Expr::AggregateFunction { fun, distinct, .. } => {
                 desc.push_str("AggregateFunction-");
@@ -779,7 +783,7 @@ mod test {
 
         let expected = "Aggregate: groupBy=[[]], aggr=[[SUM(#BinaryExpr-*BinaryExpr--Column-test.bLiteral1Column-test.a AS test.a * Int32(1) - test.b), SUM(#BinaryExpr-*BinaryExpr--Column-test.bLiteral1Column-test.a AS test.a * Int32(1) - test.b * Int32(1) + #test.c)]]\
         \n  Projection: #test.a * Int32(1) - #test.b AS BinaryExpr-*BinaryExpr--Column-test.bLiteral1Column-test.a, #test.a, #test.b, #test.c\
-        \n    TableScan: test projection=None";
+        \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -802,7 +806,7 @@ mod test {
 
         let expected = "Aggregate: groupBy=[[]], aggr=[[Int32(1) + #AggregateFunction-AVGfalseColumn-test.a AS AVG(test.a), Int32(1) - #AggregateFunction-AVGfalseColumn-test.a AS AVG(test.a)]]\
         \n  Projection: AVG(#test.a) AS AggregateFunction-AVGfalseColumn-test.a, #test.a, #test.b, #test.c\
-        \n    TableScan: test projection=None";
+        \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -822,7 +826,7 @@ mod test {
 
         let expected = "Projection: #BinaryExpr-+Column-test.aLiteral1 AS Int32(1) + test.a AS first, #BinaryExpr-+Column-test.aLiteral1 AS Int32(1) + test.a AS second\
         \n  Projection: Int32(1) + #test.a AS BinaryExpr-+Column-test.aLiteral1, #test.a, #test.b, #test.c\
-        \n    TableScan: test projection=None";
+        \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -841,7 +845,7 @@ mod test {
             .build()?;
 
         let expected = "Projection: Int32(1) + #test.a, #test.a + Int32(1)\
-        \n  TableScan: test projection=None";
+        \n  TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -859,10 +863,31 @@ mod test {
 
         let expected = "Projection: #Int32(1) + test.a\
         \n  Projection: Int32(1) + #test.a\
-        \n    TableScan: test projection=None";
+        \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
 
         Ok(())
     }
+
+    #[test]
+    fn redundant_project_fields() {
+        let table_scan = test_table_scan().unwrap();
+        let affected_id: HashSet<Identifier> =
+            ["c+a".to_string(), "d+a".to_string()].into_iter().collect();
+        let expr_set = [
+            ("c+a".to_string(), (col("c+a"), 1, DataType::UInt32)),
+            ("d+a".to_string(), (col("d+a"), 1, DataType::UInt32)),
+        ]
+        .into_iter()
+        .collect();
+        let project =
+            build_project_plan(table_scan, affected_id.clone(), &expr_set).unwrap();
+        let project_2 = build_project_plan(project, affected_id, &expr_set).unwrap();
+
+        let mut field_set = HashSet::new();
+        for field in project_2.schema().fields() {
+            assert!(field_set.insert(field.qualified_name()));
+        }
+    }
 }
