@@ -18,7 +18,7 @@ use crate::{utils, OptimizerConfig, OptimizerRule};
 use datafusion_common::{Column, DFSchema, Result};
 use datafusion_expr::{
     col,
-    expr_rewriter::replace_col,
+    expr_rewriter::{replace_col, ExprRewritable, ExprRewriter},
     logical_plan::{
         Aggregate, CrossJoin, Filter, Join, JoinType, Limit, LogicalPlan, Projection,
         TableScan, Union,
@@ -393,7 +393,7 @@ fn optimize(plan: &LogicalPlan, mut state: State) -> Result<LogicalPlan> {
             // re-write all filters based on this projection
             // E.g. in `Filter: #b\n  Projection: #a > 1 as b`, we can swap them, but the filter must be "#a > 1"
             for (predicate, columns) in state.filters.iter_mut() {
-                *predicate = rewrite(predicate, &projection)?;
+                *predicate = replace_cols_by_name(predicate.clone(), &projection)?;
 
                 columns.clear();
                 expr_to_columns(predicate, columns)?;
@@ -443,7 +443,7 @@ fn optimize(plan: &LogicalPlan, mut state: State) -> Result<LogicalPlan> {
             // rewriting predicate expressions using unqualified names as replacements
             if !projection.is_empty() {
                 for (predicate, columns) in state.filters.iter_mut() {
-                    *predicate = rewrite(predicate, &projection)?;
+                    *predicate = replace_cols_by_name(predicate.clone(), &projection)?;
 
                     columns.clear();
                     expr_to_columns(predicate, columns)?;
@@ -629,21 +629,25 @@ impl FilterPushDown {
 }
 
 /// replaces columns by its name on the projection.
-fn rewrite(expr: &Expr, projection: &HashMap<String, Expr>) -> Result<Expr> {
-    let expressions = utils::expr_sub_expressions(expr)?;
+fn replace_cols_by_name(e: Expr, replace_map: &HashMap<String, Expr>) -> Result<Expr> {
+    struct ColumnReplacer<'a> {
+        replace_map: &'a HashMap<String, Expr>,
+    }
 
-    let expressions = expressions
-        .iter()
-        .map(|e| rewrite(e, projection))
-        .collect::<Result<Vec<_>>>()?;
-
-    if let Expr::Column(c) = expr {
-        if let Some(expr) = projection.get(&c.flat_name()) {
-            return Ok(expr.clone());
+    impl<'a> ExprRewriter for ColumnReplacer<'a> {
+        fn mutate(&mut self, expr: Expr) -> Result<Expr> {
+            if let Expr::Column(c) = &expr {
+                match self.replace_map.get(&c.flat_name()) {
+                    Some(new_c) => Ok(new_c.clone()),
+                    None => Ok(expr),
+                }
+            } else {
+                Ok(expr)
+            }
         }
     }
 
-    utils::rewrite_expression(expr, &expressions)
+    e.rewrite(&mut ColumnReplacer { replace_map })
 }
 
 #[cfg(test)]
@@ -654,7 +658,7 @@ mod tests {
     use async_trait::async_trait;
     use datafusion_common::DFSchema;
     use datafusion_expr::{
-        and, col, lit,
+        and, col, in_list, in_subquery, lit,
         logical_plan::{builder::union_with_alias, JoinType},
         sum, Expr, LogicalPlanBuilder, Operator, TableSource, TableType,
     };
@@ -683,7 +687,7 @@ mod tests {
         let expected = "\
             Projection: #test.a, #test.b\
             \n  Filter: #test.a = Int64(1)\
-            \n    TableScan: test projection=None";
+            \n    TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -701,7 +705,7 @@ mod tests {
             Filter: #test.a = Int64(1)\
             \n  Limit: skip=None, fetch=10\
             \n    Projection: #test.a, #test.b\
-            \n      TableScan: test projection=None";
+            \n      TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -714,7 +718,7 @@ mod tests {
             .build()?;
         let expected = "\
             Filter: Int64(0) = Int64(1)\
-            \n  TableScan: test projection=None";
+            \n  TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -732,7 +736,7 @@ mod tests {
             Projection: #test.c, #test.b\
             \n  Projection: #test.a, #test.b, #test.c\
             \n    Filter: #test.a = Int64(1)\
-            \n      TableScan: test projection=None";
+            \n      TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -748,7 +752,7 @@ mod tests {
         let expected = "\
             Aggregate: groupBy=[[#test.a]], aggr=[[SUM(#test.b) AS total_salary]]\
             \n  Filter: #test.a > Int64(10)\
-            \n    TableScan: test projection=None";
+            \n    TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -764,7 +768,7 @@ mod tests {
         let expected = "\
             Filter: #b > Int64(10)\
             \n  Aggregate: groupBy=[[#test.a]], aggr=[[SUM(#test.b) AS b]]\
-            \n    TableScan: test projection=None";
+            \n    TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -781,7 +785,7 @@ mod tests {
         let expected = "\
             Projection: #test.a AS b, #test.c\
             \n  Filter: #test.a = Int64(1)\
-            \n    TableScan: test projection=None";
+            \n    TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -820,14 +824,14 @@ mod tests {
             "\
             Filter: #b = Int64(1)\
             \n  Projection: #test.a * Int32(2) + #test.c AS b, #test.c\
-            \n    TableScan: test projection=None"
+            \n    TableScan: test"
         );
 
         // filter is before projection
         let expected = "\
             Projection: #test.a * Int32(2) + #test.c AS b, #test.c\
             \n  Filter: #test.a * Int32(2) + #test.c = Int64(1)\
-            \n    TableScan: test projection=None";
+            \n    TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -853,7 +857,7 @@ mod tests {
             Filter: #a = Int64(1)\
             \n  Projection: #b * Int32(3) AS a, #test.c\
             \n    Projection: #test.a * Int32(2) + #test.c AS b, #test.c\
-            \n      TableScan: test projection=None"
+            \n      TableScan: test"
         );
 
         // filter is before the projections
@@ -861,7 +865,7 @@ mod tests {
         Projection: #b * Int32(3) AS a, #test.c\
         \n  Projection: #test.a * Int32(2) + #test.c AS b, #test.c\
         \n    Filter: #test.a * Int32(2) + #test.c * Int32(3) = Int64(1)\
-        \n      TableScan: test projection=None";
+        \n      TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -887,7 +891,7 @@ mod tests {
             \n  Filter: #b > Int64(10)\
             \n    Aggregate: groupBy=[[#b]], aggr=[[SUM(#test.c)]]\
             \n      Projection: #test.a AS b, #test.c\
-            \n        TableScan: test projection=None"
+            \n        TableScan: test"
         );
 
         // filter is before the projections
@@ -896,7 +900,7 @@ mod tests {
         \n  Aggregate: groupBy=[[#b]], aggr=[[SUM(#test.c)]]\
         \n    Projection: #test.a AS b, #test.c\
         \n      Filter: #test.a > Int64(10)\
-        \n        TableScan: test projection=None";
+        \n        TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
 
         Ok(())
@@ -924,7 +928,7 @@ mod tests {
             Filter: #SUM(test.c) > Int64(10) AND #b > Int64(10) AND #SUM(test.c) < Int64(20)\
             \n  Aggregate: groupBy=[[#b]], aggr=[[SUM(#test.c)]]\
             \n    Projection: #test.a AS b, #test.c\
-            \n      TableScan: test projection=None"
+            \n      TableScan: test"
         );
 
         // filter is before the projections
@@ -933,7 +937,7 @@ mod tests {
         \n  Aggregate: groupBy=[[#b]], aggr=[[SUM(#test.c)]]\
         \n    Projection: #test.a AS b, #test.c\
         \n      Filter: #test.a > Int64(10)\
-        \n        TableScan: test projection=None";
+        \n        TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
 
         Ok(())
@@ -957,7 +961,7 @@ mod tests {
             \n    Limit: skip=None, fetch=10\
             \n      Limit: skip=None, fetch=20\
             \n        Projection: #test.a, #test.b\
-            \n          TableScan: test projection=None";
+            \n          TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -973,9 +977,9 @@ mod tests {
         let expected = "\
             Union\
             \n  Filter: #a = Int64(1)\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n  Filter: #a = Int64(1)\
-            \n    TableScan: test projection=None";
+            \n    TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -994,9 +998,9 @@ mod tests {
         let expected = "\
             Union\
             \n  Filter: #a = Int64(1)\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n  Filter: #a = Int64(1)\
-            \n    TableScan: test projection=None";
+            \n    TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1022,7 +1026,7 @@ mod tests {
              \n    Limit: skip=None, fetch=1\
              \n      Filter: #test.a <= Int64(1)\
              \n        Projection: #test.a\
-             \n          TableScan: test projection=None"
+             \n          TableScan: test"
         );
 
         let expected = "\
@@ -1031,7 +1035,7 @@ mod tests {
         \n    Limit: skip=None, fetch=1\
         \n      Projection: #test.a\
         \n        Filter: #test.a <= Int64(1)\
-        \n          TableScan: test projection=None";
+        \n          TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
@@ -1055,14 +1059,14 @@ mod tests {
             \n  Filter: #test.a >= Int64(1)\
             \n    Filter: #test.a <= Int64(1)\
             \n      Limit: skip=None, fetch=1\
-            \n        TableScan: test projection=None"
+            \n        TableScan: test"
         );
 
         let expected = "\
         Projection: #test.a\
         \n  Filter: #test.a >= Int64(1) AND #test.a <= Int64(1)\
         \n    Limit: skip=None, fetch=1\
-        \n      TableScan: test projection=None";
+        \n      TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
@@ -1082,7 +1086,7 @@ mod tests {
         let expected = "\
             TestUserDefined\
              \n  Filter: #test.a <= Int64(1)\
-             \n    TableScan: test projection=None";
+             \n    TableScan: test";
 
         // not part of the test
         assert_eq!(format!("{:?}", plan), expected);
@@ -1116,19 +1120,19 @@ mod tests {
             "\
             Filter: #test.a <= Int64(1)\
             \n  Inner Join: #test.a = #test2.a\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n    Projection: #test2.a\
-            \n      TableScan: test2 projection=None"
+            \n      TableScan: test2"
         );
 
         // filter sent to side before the join
         let expected = "\
         Inner Join: #test.a = #test2.a\
         \n  Filter: #test.a <= Int64(1)\
-        \n    TableScan: test projection=None\
+        \n    TableScan: test\
         \n  Projection: #test2.a\
         \n    Filter: #test2.a <= Int64(1)\
-        \n      TableScan: test2 projection=None";
+        \n      TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1157,19 +1161,19 @@ mod tests {
             "\
             Filter: #test.a <= Int64(1)\
             \n  Inner Join: Using #test.a = #test2.a\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n    Projection: #test2.a\
-            \n      TableScan: test2 projection=None"
+            \n      TableScan: test2"
         );
 
         // filter sent to side before the join
         let expected = "\
         Inner Join: Using #test.a = #test2.a\
         \n  Filter: #test.a <= Int64(1)\
-        \n    TableScan: test projection=None\
+        \n    TableScan: test\
         \n  Projection: #test2.a\
         \n    Filter: #test2.a <= Int64(1)\
-        \n      TableScan: test2 projection=None";
+        \n      TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1203,9 +1207,9 @@ mod tests {
             Filter: #test.c <= #test2.b\
             \n  Inner Join: #test.a = #test2.a\
             \n    Projection: #test.a, #test.c\
-            \n      TableScan: test projection=None\
+            \n      TableScan: test\
             \n    Projection: #test2.a, #test2.b\
-            \n      TableScan: test2 projection=None"
+            \n      TableScan: test2"
         );
 
         // expected is equal: no push-down
@@ -1243,18 +1247,18 @@ mod tests {
             Filter: #test.b <= Int64(1)\
             \n  Inner Join: #test.a = #test2.a\
             \n    Projection: #test.a, #test.b\
-            \n      TableScan: test projection=None\
+            \n      TableScan: test\
             \n    Projection: #test2.a, #test2.c\
-            \n      TableScan: test2 projection=None"
+            \n      TableScan: test2"
         );
 
         let expected = "\
         Inner Join: #test.a = #test2.a\
         \n  Projection: #test.a, #test.b\
         \n    Filter: #test.b <= Int64(1)\
-        \n      TableScan: test projection=None\
+        \n      TableScan: test\
         \n  Projection: #test2.a, #test2.c\
-        \n    TableScan: test2 projection=None";
+        \n    TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1284,18 +1288,18 @@ mod tests {
             "\
             Filter: #test2.a <= Int64(1)\
             \n  Left Join: Using #test.a = #test2.a\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n    Projection: #test2.a\
-            \n      TableScan: test2 projection=None"
+            \n      TableScan: test2"
         );
 
         // filter not duplicated nor pushed down - i.e. noop
         let expected = "\
         Filter: #test2.a <= Int64(1)\
         \n  Left Join: Using #test.a = #test2.a\
-        \n    TableScan: test projection=None\
+        \n    TableScan: test\
         \n    Projection: #test2.a\
-        \n      TableScan: test2 projection=None";
+        \n      TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1325,18 +1329,18 @@ mod tests {
             "\
             Filter: #test.a <= Int64(1)\
             \n  Right Join: Using #test.a = #test2.a\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n    Projection: #test2.a\
-            \n      TableScan: test2 projection=None"
+            \n      TableScan: test2"
         );
 
         // filter not duplicated nor pushed down - i.e. noop
         let expected = "\
         Filter: #test.a <= Int64(1)\
         \n  Right Join: Using #test.a = #test2.a\
-        \n    TableScan: test projection=None\
+        \n    TableScan: test\
         \n    Projection: #test2.a\
-        \n      TableScan: test2 projection=None";
+        \n      TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1366,18 +1370,18 @@ mod tests {
             "\
             Filter: #test.a <= Int64(1)\
             \n  Left Join: Using #test.a = #test2.a\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n    Projection: #test2.a\
-            \n      TableScan: test2 projection=None"
+            \n      TableScan: test2"
         );
 
         // filter sent to left side of the join, not the right
         let expected = "\
         Left Join: Using #test.a = #test2.a\
         \n  Filter: #test.a <= Int64(1)\
-        \n    TableScan: test projection=None\
+        \n    TableScan: test\
         \n  Projection: #test2.a\
-        \n    TableScan: test2 projection=None";
+        \n    TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1407,18 +1411,18 @@ mod tests {
             "\
             Filter: #test2.a <= Int64(1)\
             \n  Right Join: Using #test.a = #test2.a\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n    Projection: #test2.a\
-            \n      TableScan: test2 projection=None"
+            \n      TableScan: test2"
         );
 
         // filter sent to right side of join, not duplicated to the left
         let expected = "\
         Right Join: Using #test.a = #test2.a\
-        \n  TableScan: test projection=None\
+        \n  TableScan: test\
         \n  Projection: #test2.a\
         \n    Filter: #test2.a <= Int64(1)\
-        \n      TableScan: test2 projection=None";
+        \n      TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1453,19 +1457,19 @@ mod tests {
             "\
             Inner Join: #test.a = #test2.a Filter: #test.c > UInt32(1) AND #test.b < #test2.b AND #test2.c > UInt32(4)\
             \n  Projection: #test.a, #test.b, #test.c\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n  Projection: #test2.a, #test2.b, #test2.c\
-            \n    TableScan: test2 projection=None"
+            \n    TableScan: test2"
         );
 
         let expected = "\
         Inner Join: #test.a = #test2.a Filter: #test.b < #test2.b\
         \n  Projection: #test.a, #test.b, #test.c\
         \n    Filter: #test.c > UInt32(1)\
-        \n      TableScan: test projection=None\
+        \n      TableScan: test\
         \n  Projection: #test2.a, #test2.b, #test2.c\
         \n    Filter: #test2.c > UInt32(4)\
-        \n      TableScan: test2 projection=None";
+        \n      TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1499,19 +1503,19 @@ mod tests {
             "\
             Inner Join: #test.a = #test2.a Filter: #test.b > UInt32(1) AND #test2.c > UInt32(4)\
             \n  Projection: #test.a, #test.b, #test.c\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n  Projection: #test2.a, #test2.b, #test2.c\
-            \n    TableScan: test2 projection=None"
+            \n    TableScan: test2"
         );
 
         let expected = "\
         Inner Join: #test.a = #test2.a\
         \n  Projection: #test.a, #test.b, #test.c\
         \n    Filter: #test.b > UInt32(1)\
-        \n      TableScan: test projection=None\
+        \n      TableScan: test\
         \n  Projection: #test2.a, #test2.b, #test2.c\
         \n    Filter: #test2.c > UInt32(4)\
-        \n      TableScan: test2 projection=None";
+        \n      TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1543,19 +1547,19 @@ mod tests {
             "\
             Inner Join: #test.a = #test2.b Filter: #test.a > UInt32(1)\
             \n  Projection: #test.a\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n  Projection: #test2.b\
-            \n    TableScan: test2 projection=None"
+            \n    TableScan: test2"
         );
 
         let expected = "\
         Inner Join: #test.a = #test2.b\
         \n  Projection: #test.a\
         \n    Filter: #test.a > UInt32(1)\
-        \n      TableScan: test projection=None\
+        \n      TableScan: test\
         \n  Projection: #test2.b\
         \n    Filter: #test2.b > UInt32(1)\
-        \n      TableScan: test2 projection=None";
+        \n      TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1590,18 +1594,18 @@ mod tests {
             "\
             Left Join: #test.a = #test2.a Filter: #test.a > UInt32(1) AND #test.b < #test2.b AND #test2.c > UInt32(4)\
             \n  Projection: #test.a, #test.b, #test.c\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n  Projection: #test2.a, #test2.b, #test2.c\
-            \n    TableScan: test2 projection=None"
+            \n    TableScan: test2"
         );
 
         let expected = "\
         Left Join: #test.a = #test2.a Filter: #test.a > UInt32(1) AND #test.b < #test2.b\
         \n  Projection: #test.a, #test.b, #test.c\
-        \n    TableScan: test projection=None\
+        \n    TableScan: test\
         \n  Projection: #test2.a, #test2.b, #test2.c\
         \n    Filter: #test2.c > UInt32(4)\
-        \n      TableScan: test2 projection=None";
+        \n      TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1636,18 +1640,18 @@ mod tests {
             "\
             Right Join: #test.a = #test2.a Filter: #test.a > UInt32(1) AND #test.b < #test2.b AND #test2.c > UInt32(4)\
             \n  Projection: #test.a, #test.b, #test.c\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n  Projection: #test2.a, #test2.b, #test2.c\
-            \n    TableScan: test2 projection=None"
+            \n    TableScan: test2"
         );
 
         let expected = "\
         Right Join: #test.a = #test2.a Filter: #test.b < #test2.b AND #test2.c > UInt32(4)\
         \n  Projection: #test.a, #test.b, #test.c\
         \n    Filter: #test.a > UInt32(1)\
-        \n      TableScan: test projection=None\
+        \n      TableScan: test\
         \n  Projection: #test2.a, #test2.b, #test2.c\
-        \n    TableScan: test2 projection=None";
+        \n    TableScan: test2";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1682,9 +1686,9 @@ mod tests {
             "\
             Full Join: #test.a = #test2.a Filter: #test.a > UInt32(1) AND #test.b < #test2.b AND #test2.c > UInt32(4)\
             \n  Projection: #test.a, #test.b, #test.c\
-            \n    TableScan: test projection=None\
+            \n    TableScan: test\
             \n  Projection: #test2.a, #test2.b, #test2.c\
-            \n    TableScan: test2 projection=None"
+            \n    TableScan: test2"
         );
 
         let expected = &format!("{:?}", plan);
@@ -1755,7 +1759,7 @@ mod tests {
         let plan = table_scan_with_pushdown_provider(TableProviderFilterPushDown::Exact)?;
 
         let expected = "\
-        TableScan: test projection=None, full_filters=[#a = Int64(1)]";
+        TableScan: test, full_filters=[#a = Int64(1)]";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1767,7 +1771,7 @@ mod tests {
 
         let expected = "\
         Filter: #a = Int64(1)\
-        \n  TableScan: test projection=None, partial_filters=[#a = Int64(1)]";
+        \n  TableScan: test, partial_filters=[#a = Int64(1)]";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1781,7 +1785,7 @@ mod tests {
 
         let expected = "\
         Filter: #a = Int64(1)\
-        \n  TableScan: test projection=None, partial_filters=[#a = Int64(1)]";
+        \n  TableScan: test, partial_filters=[#a = Int64(1)]";
 
         // Optimizing the same plan multiple times should produce the same plan
         // each time.
@@ -1796,7 +1800,7 @@ mod tests {
 
         let expected = "\
         Filter: #a = Int64(1)\
-        \n  TableScan: test projection=None";
+        \n  TableScan: test";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -1825,10 +1829,257 @@ mod tests {
 
         let expected ="Projection: #a, #b\
             \n  Filter: #a = Int64(10) AND #b > Int64(11)\
-            \n    TableScan: test projection=Some([a]), partial_filters=[#a = Int64(10), #b > Int64(11)]";
+            \n    TableScan: test projection=[a], partial_filters=[#a = Int64(10), #b > Int64(11)]";
 
         assert_optimized_plan_eq(&plan, expected);
 
         Ok(())
     }
+
+    #[test]
+    fn test_filter_with_alias() -> Result<()> {
+        // in table scan the true col name is 'test.a',
+        // but we rename it as 'b', and use col 'b' in filter
+        // we need rewrite filter col before push down.
+        let table_scan = test_table_scan()?;
+        let plan = LogicalPlanBuilder::from(table_scan)
+            .project(vec![col("a").alias("b"), col("c")])?
+            .filter(and(col("b").gt(lit(10i64)), col("c").gt(lit(10i64))))?
+            .build()?;
+
+        // filter on col b
+        assert_eq!(
+            format!("{:?}", plan),
+            "\
+            Filter: #b > Int64(10) AND #test.c > Int64(10)\
+            \n  Projection: #test.a AS b, #test.c\
+            \n    TableScan: test\
+            "
+        );
+
+        // rewrite filter col b to test.a
+        let expected = "\
+            Projection: #test.a AS b, #test.c\
+            \n  Filter: #test.a > Int64(10) AND #test.c > Int64(10)\
+            \n    TableScan: test\
+            ";
+
+        assert_optimized_plan_eq(&plan, expected);
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_filter_with_alias_2() -> Result<()> {
+        // in table scan the true col name is 'test.a',
+        // but we rename it as 'b', and use col 'b' in filter
+        // we need rewrite filter col before push down.
+        let table_scan = test_table_scan()?;
+        let plan = LogicalPlanBuilder::from(table_scan)
+            .project(vec![col("a").alias("b"), col("c")])?
+            .project(vec![col("b"), col("c")])?
+            .filter(and(col("b").gt(lit(10i64)), col("c").gt(lit(10i64))))?
+            .build()?;
+
+        // filter on col b
+        assert_eq!(
+            format!("{:?}", plan),
+            "\
+            Filter: #b > Int64(10) AND #test.c > Int64(10)\
+            \n  Projection: #b, #test.c\
+            \n    Projection: #test.a AS b, #test.c\
+            \n      TableScan: test\
+            "
+        );
+
+        // rewrite filter col b to test.a
+        let expected = "\
+            Projection: #b, #test.c\
+            \n  Projection: #test.a AS b, #test.c\
+            \n    Filter: #test.a > Int64(10) AND #test.c > Int64(10)\
+            \n      TableScan: test\
+            ";
+
+        assert_optimized_plan_eq(&plan, expected);
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_filter_with_multi_alias() -> Result<()> {
+        let table_scan = test_table_scan()?;
+        let plan = LogicalPlanBuilder::from(table_scan)
+            .project(vec![col("a").alias("b"), col("c").alias("d")])?
+            .filter(and(col("b").gt(lit(10i64)), col("d").gt(lit(10i64))))?
+            .build()?;
+
+        // filter on col b and d
+        assert_eq!(
+            format!("{:?}", plan),
+            "\
+            Filter: #b > Int64(10) AND #d > Int64(10)\
+            \n  Projection: #test.a AS b, #test.c AS d\
+            \n    TableScan: test\
+            "
+        );
+
+        // rewrite filter col b to test.a, col d to test.c
+        let expected = "\
+            Projection: #test.a AS b, #test.c AS d\
+            \n  Filter: #test.a > Int64(10) AND #test.c > Int64(10)\
+            \n    TableScan: test\
+            ";
+
+        assert_optimized_plan_eq(&plan, expected);
+
+        Ok(())
+    }
+
+    /// predicate on join key in filter expression should be pushed down to both inputs
+    #[test]
+    fn join_filter_with_alias() -> Result<()> {
+        let table_scan = test_table_scan()?;
+        let left = LogicalPlanBuilder::from(table_scan)
+            .project(vec![col("a").alias("c")])?
+            .build()?;
+        let right_table_scan = test_table_scan_with_name("test2")?;
+        let right = LogicalPlanBuilder::from(right_table_scan)
+            .project(vec![col("b").alias("d")])?
+            .build()?;
+        let filter = col("c").gt(lit(1u32));
+        let plan = LogicalPlanBuilder::from(left)
+            .join(
+                &right,
+                JoinType::Inner,
+                (vec![Column::from_name("c")], vec![Column::from_name("d")]),
+                Some(filter),
+            )?
+            .build()?;
+
+        assert_eq!(
+            format!("{:?}", plan),
+            "\
+            Inner Join: #c = #d Filter: #c > UInt32(1)\
+            \n  Projection: #test.a AS c\
+            \n    TableScan: test\
+            \n  Projection: #test2.b AS d\
+            \n    TableScan: test2"
+        );
+
+        // Change filter on col `c`, 'd' to `test.a`, 'test.b'
+        let expected = "\
+        Inner Join: #c = #d\
+        \n  Projection: #test.a AS c\
+        \n    Filter: #test.a > UInt32(1)\
+        \n      TableScan: test\
+        \n  Projection: #test2.b AS d\
+        \n    Filter: #test2.b > UInt32(1)\
+        \n      TableScan: test2";
+        assert_optimized_plan_eq(&plan, expected);
+        Ok(())
+    }
+
+    #[test]
+    fn test_in_filter_with_alias() -> Result<()> {
+        // in table scan the true col name is 'test.a',
+        // but we rename it as 'b', and use col 'b' in filter
+        // we need rewrite filter col before push down.
+        let table_scan = test_table_scan()?;
+        let filter_value = vec![lit(1u32), lit(2u32), lit(3u32), lit(4u32)];
+        let plan = LogicalPlanBuilder::from(table_scan)
+            .project(vec![col("a").alias("b"), col("c")])?
+            .filter(in_list(col("b"), filter_value, false))?
+            .build()?;
+
+        // filter on col b
+        assert_eq!(
+            format!("{:?}", plan),
+            "\
+            Filter: #b IN ([UInt32(1), UInt32(2), UInt32(3), UInt32(4)])\
+            \n  Projection: #test.a AS b, #test.c\
+            \n    TableScan: test\
+            "
+        );
+
+        // rewrite filter col b to test.a
+        let expected = "\
+            Projection: #test.a AS b, #test.c\
+            \n  Filter: #test.a IN ([UInt32(1), UInt32(2), UInt32(3), UInt32(4)])\
+            \n    TableScan: test\
+            ";
+
+        assert_optimized_plan_eq(&plan, expected);
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_in_filter_with_alias_2() -> Result<()> {
+        // in table scan the true col name is 'test.a',
+        // but we rename it as 'b', and use col 'b' in filter
+        // we need rewrite filter col before push down.
+        let table_scan = test_table_scan()?;
+        let filter_value = vec![lit(1u32), lit(2u32), lit(3u32), lit(4u32)];
+        let plan = LogicalPlanBuilder::from(table_scan)
+            .project(vec![col("a").alias("b"), col("c")])?
+            .project(vec![col("b"), col("c")])?
+            .filter(in_list(col("b"), filter_value, false))?
+            .build()?;
+
+        // filter on col b
+        assert_eq!(
+            format!("{:?}", plan),
+            "\
+            Filter: #b IN ([UInt32(1), UInt32(2), UInt32(3), UInt32(4)])\
+            \n  Projection: #b, #test.c\
+            \n    Projection: #test.a AS b, #test.c\
+            \n      TableScan: test\
+            "
+        );
+
+        // rewrite filter col b to test.a
+        let expected = "\
+            Projection: #b, #test.c\
+            \n  Projection: #test.a AS b, #test.c\
+            \n    Filter: #test.a IN ([UInt32(1), UInt32(2), UInt32(3), UInt32(4)])\
+            \n      TableScan: test\
+            ";
+
+        assert_optimized_plan_eq(&plan, expected);
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_in_subquery_with_alias() -> Result<()> {
+        // in table scan the true col name is 'test.a',
+        // but we rename it as 'b', and use col 'b' in subquery filter
+        let table_scan = test_table_scan()?;
+        let table_scan_sq = test_table_scan_with_name("sq")?;
+        let subplan = Arc::new(
+            LogicalPlanBuilder::from(table_scan_sq)
+                .project(vec![col("c")])?
+                .build()?,
+        );
+        let plan = LogicalPlanBuilder::from(table_scan)
+            .project(vec![col("a").alias("b"), col("c")])?
+            .filter(in_subquery(col("b"), subplan))?
+            .build()?;
+
+        // filter on col b in subquery
+        let expected_before = "\
+        Filter: #b IN (Subquery: Projection: #sq.c\n  TableScan: sq)\
+        \n  Projection: #test.a AS b, #test.c\
+        \n    TableScan: test";
+        assert_eq!(format!("{:?}", plan), expected_before);
+
+        // rewrite filter col b to test.a
+        let expected_after = "\
+        Projection: #test.a AS b, #test.c\
+        \n  Filter: #test.a IN (Subquery: Projection: #sq.c\n  TableScan: sq)\
+        \n    TableScan: test";
+        assert_optimized_plan_eq(&plan, expected_after);
+
+        Ok(())
+    }
 }
