@@ -24,6 +24,7 @@ use arrow::error::Result as ArrowResult;
 use datafusion_common::{
     Column, DFField, DFSchema, DFSchemaRef, DataFusionError, Result, ToDFSchema,
 };
+use datafusion_expr::utils::grouping_set_to_exprlist;
 use datafusion_expr::{
     logical_plan::{
         builder::{build_join_schema, LogicalPlanBuilder},
@@ -314,7 +315,10 @@ fn optimize_plan(
             // * remove any aggregate expression that is not required
             // * construct the new set of required columns
 
-            exprlist_to_columns(group_expr, &mut new_required_columns)?;
+            // Find distinct group by exprs in the case where we have a grouping set
+            let all_group_expr: Vec<Expr> = grouping_set_to_exprlist(group_expr)?;
+
+            exprlist_to_columns(&all_group_expr, &mut new_required_columns)?;
 
             // Gather all columns needed for expressions in this Aggregate
             let mut new_aggr_expr = Vec::new();
@@ -450,7 +454,7 @@ fn optimize_plan(
                 schema.metadata().clone(),
             )?;
             Ok(LogicalPlan::Union(Union {
-                inputs: new_inputs,
+                inputs: new_inputs.iter().cloned().map(Arc::new).collect(),
                 schema: Arc::new(new_schema),
                 alias: alias.clone(),
             }))
@@ -499,6 +503,7 @@ fn optimize_plan(
         | LogicalPlan::CreateCatalog(_)
         | LogicalPlan::DropTable(_)
         | LogicalPlan::CrossJoin(_)
+        | LogicalPlan::Distinct(_)
         | LogicalPlan::Extension { .. } => {
             let expr = plan.expressions();
             // collect all required columns by this plan
@@ -548,7 +553,7 @@ mod tests {
             .build()?;
 
         let expected = "Aggregate: groupBy=[[]], aggr=[[MAX(#test.b)]]\
-        \n  TableScan: test projection=Some([b])";
+        \n  TableScan: test projection=[b]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -564,7 +569,7 @@ mod tests {
             .build()?;
 
         let expected = "Aggregate: groupBy=[[#test.c]], aggr=[[MAX(#test.b)]]\
-        \n  TableScan: test projection=Some([b, c])";
+        \n  TableScan: test projection=[b, c]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -582,7 +587,7 @@ mod tests {
 
         let expected = "Aggregate: groupBy=[[#a.c]], aggr=[[MAX(#a.b)]]\
         \n  SubqueryAlias: a\
-        \n    TableScan: test projection=Some([b, c])";
+        \n    TableScan: test projection=[b, c]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -600,7 +605,7 @@ mod tests {
 
         let expected = "Aggregate: groupBy=[[]], aggr=[[MAX(#test.b)]]\
         \n  Filter: #test.c\
-        \n    TableScan: test projection=Some([b, c])";
+        \n    TableScan: test projection=[b, c]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -616,7 +621,7 @@ mod tests {
             .project(vec![col("a"), col("c"), col("b")])?
             .build()?;
         let expected = "Projection: #test.a, #test.c, #test.b\
-        \n  TableScan: test projection=Some([a, b, c])";
+        \n  TableScan: test projection=[a, b, c]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -631,7 +636,7 @@ mod tests {
             .project(vec![col("c"), col("b"), col("a")])?
             .build()?;
         let expected = "Projection: #test.c, #test.b, #test.a\
-        \n  TableScan: test projection=Some([a, b, c])";
+        \n  TableScan: test projection=[a, b, c]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -654,7 +659,7 @@ mod tests {
         \n  Filter: #test.a > Int32(1)\
         \n    Filter: #test.b > Int32(1)\
         \n      Filter: #test.c > Int32(1)\
-        \n        TableScan: test projection=Some([a, b, c])";
+        \n        TableScan: test projection=[a, b, c]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -676,8 +681,8 @@ mod tests {
         // make sure projections are pushed down to both table scans
         let expected = "Projection: #test.a, #test.b, #test2.c1\
         \n  Left Join: #test.a = #test2.c1\
-        \n    TableScan: test projection=Some([a, b])\
-        \n    TableScan: test2 projection=Some([c1])";
+        \n    TableScan: test projection=[a, b]\
+        \n    TableScan: test2 projection=[c1]";
 
         let optimized_plan = optimize(&plan)?;
         let formatted_plan = format!("{:?}", optimized_plan);
@@ -719,8 +724,8 @@ mod tests {
         // make sure projections are pushed down to both table scans
         let expected = "Projection: #test.a, #test.b\
         \n  Left Join: #test.a = #test2.c1\
-        \n    TableScan: test projection=Some([a, b])\
-        \n    TableScan: test2 projection=Some([c1])";
+        \n    TableScan: test projection=[a, b]\
+        \n    TableScan: test2 projection=[c1]";
 
         let optimized_plan = optimize(&plan)?;
         let formatted_plan = format!("{:?}", optimized_plan);
@@ -760,8 +765,8 @@ mod tests {
         // make sure projections are pushed down to table scan
         let expected = "Projection: #test.a, #test.b\
         \n  Left Join: Using #test.a = #test2.a\
-        \n    TableScan: test projection=Some([a, b])\
-        \n    TableScan: test2 projection=Some([a])";
+        \n    TableScan: test projection=[a, b]\
+        \n    TableScan: test2 projection=[a]";
 
         let optimized_plan = optimize(&plan)?;
         let formatted_plan = format!("{:?}", optimized_plan);
@@ -796,7 +801,7 @@ mod tests {
             .build()?;
 
         let expected = "Projection: CAST(#test.c AS Float64)\
-        \n  TableScan: test projection=Some([c])";
+        \n  TableScan: test projection=[c]";
 
         assert_optimized_plan_eq(&projection, expected);
 
@@ -816,7 +821,7 @@ mod tests {
         assert_fields_eq(&plan, vec!["a", "b"]);
 
         let expected = "Projection: #test.a, #test.b\
-        \n  TableScan: test projection=Some([a, b])";
+        \n  TableScan: test projection=[a, b]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -850,7 +855,7 @@ mod tests {
         assert_fields_eq(&plan, vec!["a", "b"]);
 
         let expected = "Projection: #a, #b\
-        \n  TableScan: test projection=Some([a, b])";
+        \n  TableScan: test projection=[a, b]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -872,7 +877,7 @@ mod tests {
 
         let expected = "Limit: skip=None, fetch=5\
         \n  Projection: #test.c, #test.a\
-        \n    TableScan: test projection=Some([a, c])";
+        \n    TableScan: test projection=[a, c]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -884,7 +889,7 @@ mod tests {
         let table_scan = test_table_scan()?;
         let plan = LogicalPlanBuilder::from(table_scan).build()?;
         // should expand projection to all columns without projection
-        let expected = "TableScan: test projection=Some([a, b, c])";
+        let expected = "TableScan: test projection=[a, b, c]";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -896,7 +901,7 @@ mod tests {
             .project(vec![lit(1_i64), lit(2_i64)])?
             .build()?;
         let expected = "Projection: Int64(1), Int64(2)\
-                      \n  TableScan: test projection=Some([a])";
+                      \n  TableScan: test projection=[a]";
         assert_optimized_plan_eq(&plan, expected);
         Ok(())
     }
@@ -921,7 +926,7 @@ mod tests {
         Aggregate: groupBy=[[#test.c]], aggr=[[MAX(#test.a)]]\
         \n  Filter: #test.c > Int32(1)\
         \n    Projection: #test.c, #test.a\
-        \n      TableScan: test projection=Some([a, c])";
+        \n      TableScan: test projection=[a, c]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -945,7 +950,7 @@ mod tests {
 
         let expected = "\
         Projection: Int32(1) AS a\
-        \n  TableScan: test projection=Some([a])";
+        \n  TableScan: test projection=[a]";
 
         assert_optimized_plan_eq(&plan, expected);
 
@@ -991,7 +996,7 @@ mod tests {
         let expected = "Projection: #test.c, #test.a, #MAX(test.b)\
         \n  Filter: #test.c > Int32(1)\
         \n    Aggregate: groupBy=[[#test.a, #test.c]], aggr=[[MAX(#test.b)]]\
-        \n      TableScan: test projection=Some([a, b, c])";
+        \n      TableScan: test projection=[a, b, c]";
 
         assert_optimized_plan_eq(&plan, expected);
 
