@@ -137,8 +137,8 @@ fn is_bool_lit(expr: &Expr) -> bool {
     matches!(expr, Expr::Literal(ScalarValue::Boolean(_)))
 }
 
-/// Return a literal NULL value
-fn lit_null() -> Expr {
+/// Return a literal NULL value of Boolean data type
+fn lit_bool_null() -> Expr {
     Expr::Literal(ScalarValue::Boolean(None))
 }
 
@@ -483,7 +483,7 @@ impl<'a, S: SimplifyInfo> ExprRewriter for Simplifier<'a, S> {
                 match as_bool_lit(*left) {
                     Some(true) => *right,
                     Some(false) => Not(right),
-                    None => lit_null(),
+                    None => lit_bool_null(),
                 }
             }
             // A = true  --> A
@@ -497,7 +497,7 @@ impl<'a, S: SimplifyInfo> ExprRewriter for Simplifier<'a, S> {
                 match as_bool_lit(*right) {
                     Some(true) => *left,
                     Some(false) => Not(left),
-                    None => lit_null(),
+                    None => lit_bool_null(),
                 }
             }
 
@@ -516,7 +516,7 @@ impl<'a, S: SimplifyInfo> ExprRewriter for Simplifier<'a, S> {
                 match as_bool_lit(*left) {
                     Some(true) => Not(right),
                     Some(false) => *right,
-                    None => lit_null(),
+                    None => lit_bool_null(),
                 }
             }
             // A != true  --> !A
@@ -530,7 +530,7 @@ impl<'a, S: SimplifyInfo> ExprRewriter for Simplifier<'a, S> {
                 match as_bool_lit(*right) {
                     Some(true) => Not(left),
                     Some(false) => *left,
-                    None => lit_null(),
+                    None => lit_bool_null(),
                 }
             }
 
@@ -996,7 +996,7 @@ mod tests {
 
     #[test]
     fn test_simplify_null_and_false() {
-        let expr = binary_expr(lit_null(), Operator::And, lit(false));
+        let expr = binary_expr(lit_bool_null(), Operator::And, lit(false));
         let expr_eq = lit(false);
 
         assert_eq!(simplify(expr), expr_eq);
@@ -1457,11 +1457,6 @@ mod tests {
         );
     }
 
-    /// Boolean null
-    fn lit_null() -> Expr {
-        Expr::Literal(ScalarValue::Boolean(None))
-    }
-
     #[test]
     fn simplify_expr_bool_or() {
         // col || true is always true
@@ -1471,16 +1466,16 @@ mod tests {
         assert_eq!(simplify(col("c2").or(lit(false))), col("c2"),);
 
         // true || null is always true
-        assert_eq!(simplify(lit(true).or(lit_null())), lit(true),);
+        assert_eq!(simplify(lit(true).or(lit_bool_null())), lit(true),);
 
         // null || true is always true
-        assert_eq!(simplify(lit_null().or(lit(true))), lit(true),);
+        assert_eq!(simplify(lit_bool_null().or(lit(true))), lit(true),);
 
         // false || null is always null
-        assert_eq!(simplify(lit(false).or(lit_null())), lit_null(),);
+        assert_eq!(simplify(lit(false).or(lit_bool_null())), lit_bool_null(),);
 
         // null || false is always null
-        assert_eq!(simplify(lit_null().or(lit(false))), lit_null(),);
+        assert_eq!(simplify(lit_bool_null().or(lit(false))), lit_bool_null(),);
 
         // ( c1 BETWEEN Int32(0) AND Int32(10) ) OR Boolean(NULL)
         // it can be either NULL or  TRUE depending on the value of `c1 BETWEEN Int32(0) AND Int32(10)`
@@ -1491,7 +1486,7 @@ mod tests {
             low: Box::new(lit(0)),
             high: Box::new(lit(10)),
         };
-        let expr = expr.or(lit_null());
+        let expr = expr.or(lit_bool_null());
         let result = simplify(expr.clone());
         assert_eq!(expr, result);
     }
@@ -1504,16 +1499,16 @@ mod tests {
         assert_eq!(simplify(col("c2").and(lit(false))), lit(false),);
 
         // true && null is always null
-        assert_eq!(simplify(lit(true).and(lit_null())), lit_null(),);
+        assert_eq!(simplify(lit(true).and(lit_bool_null())), lit_bool_null(),);
 
         // null && true is always null
-        assert_eq!(simplify(lit_null().and(lit(true))), lit_null(),);
+        assert_eq!(simplify(lit_bool_null().and(lit(true))), lit_bool_null(),);
 
         // false && null is always false
-        assert_eq!(simplify(lit(false).and(lit_null())), lit(false),);
+        assert_eq!(simplify(lit(false).and(lit_bool_null())), lit(false),);
 
         // null && false is always false
-        assert_eq!(simplify(lit_null().and(lit(false))), lit(false),);
+        assert_eq!(simplify(lit_bool_null().and(lit(false))), lit(false),);
 
         // c1 BETWEEN Int32(0) AND Int32(10) AND Boolean(NULL)
         // it can be either NULL or FALSE depending on the value of `c1 BETWEEN Int32(0) AND Int32(10`
@@ -1524,7 +1519,7 @@ mod tests {
             low: Box::new(lit(0)),
             high: Box::new(lit(10)),
         };
-        let expr = expr.and(lit_null());
+        let expr = expr.and(lit_bool_null());
         let result = simplify(expr.clone());
         assert_eq!(expr, result);
     }
@@ -1572,7 +1567,7 @@ mod tests {
             "\
 	        Filter: #test.b > Int32(1) AS test.b > Int32(1) AND test.b > Int32(1)\
             \n  Projection: #test.a\
-            \n    TableScan: test projection=None",
+            \n    TableScan: test",
         );
     }
 
@@ -1596,7 +1591,7 @@ mod tests {
             "\
             Filter: #test.a > Int32(5) AND #test.b < Int32(6) AS test.a > Int32(5) AND test.b < Int32(6) AND test.a > Int32(5)\
             \n  Projection: #test.a, #test.b\
-	        \n    TableScan: test projection=None",
+	        \n    TableScan: test",
         );
     }
 
@@ -1617,7 +1612,7 @@ mod tests {
         Projection: #test.a\
         \n  Filter: NOT #test.c AS test.c = Boolean(false)\
         \n    Filter: #test.b AS test.b = Boolean(true)\
-        \n      TableScan: test projection=None";
+        \n      TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
     }
@@ -1642,7 +1637,7 @@ mod tests {
         \n  Limit: skip=None, fetch=1\
         \n    Filter: #test.c AS test.c != Boolean(false)\
         \n      Filter: NOT #test.b AS test.b != Boolean(true)\
-        \n        TableScan: test projection=None";
+        \n        TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
     }
@@ -1661,7 +1656,7 @@ mod tests {
         let expected = "\
         Projection: #test.a\
         \n  Filter: NOT #test.b AND #test.c AS test.b != Boolean(true) AND test.c = Boolean(true)\
-        \n    TableScan: test projection=None";
+        \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
     }
@@ -1680,7 +1675,7 @@ mod tests {
         let expected = "\
         Projection: #test.a\
         \n  Filter: NOT #test.b OR NOT #test.c AS test.b != Boolean(true) OR test.c = Boolean(false)\
-        \n    TableScan: test projection=None";
+        \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
     }
@@ -1699,7 +1694,7 @@ mod tests {
         let expected = "\
         Projection: #test.a\
         \n  Filter: #test.b AS NOT test.b = Boolean(false)\
-        \n    TableScan: test projection=None";
+        \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
     }
@@ -1715,7 +1710,7 @@ mod tests {
 
         let expected = "\
         Projection: #test.a, #test.d, NOT #test.b AS test.b = Boolean(false)\
-        \n  TableScan: test projection=None";
+        \n  TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
     }
@@ -1740,7 +1735,7 @@ mod tests {
         let expected = "\
         Aggregate: groupBy=[[#test.a, #test.c]], aggr=[[MAX(#test.b) AS MAX(test.b = Boolean(true)), MIN(#test.b)]]\
         \n  Projection: #test.a, #test.c, #test.b\
-        \n    TableScan: test projection=None";
+        \n    TableScan: test";
 
         assert_optimized_plan_eq(&plan, expected);
     }
@@ -1805,7 +1800,7 @@ mod tests {
             .unwrap();
 
         let expected = "Projection: TimestampNanosecond(1599566400000000000, None) AS totimestamp(Utf8(\"2020-09-08T12:00:00+00:00\"))\
-            \n  TableScan: test projection=None"
+            \n  TableScan: test"
             .to_string();
         let actual = get_optimized_plan_formatted(&plan, &Utc::now());
         assert_eq!(expected, actual);
@@ -1840,7 +1835,7 @@ mod tests {
             .unwrap();
 
         let expected = "Projection: Int32(0) AS CAST(Utf8(\"0\") AS Int32)\
-            \n  TableScan: test projection=None";
+            \n  TableScan: test";
         let actual = get_optimized_plan_formatted(&plan, &Utc::now());
         assert_eq!(expected, actual);
     }
@@ -1882,7 +1877,7 @@ mod tests {
         let actual = get_optimized_plan_formatted(&plan, &time);
         let expected = format!(
             "Projection: TimestampNanosecond({}, Some(\"UTC\")) AS now(), TimestampNanosecond({}, Some(\"UTC\")) AS t2\
-            \n  TableScan: test projection=None",
+            \n  TableScan: test",
             time.timestamp_nanos(),
             time.timestamp_nanos()
         );
@@ -1907,7 +1902,7 @@ mod tests {
         let actual = get_optimized_plan_formatted(&plan, &time);
         let expected =
             "Projection: NOT #test.a AS Boolean(true) OR Boolean(false) != test.a\
-                        \n  TableScan: test projection=None";
+                        \n  TableScan: test";
 
         assert_eq!(actual, expected);
     }
@@ -1932,7 +1927,7 @@ mod tests {
         // Note that constant folder runs and folds the entire
         // expression down to a single constant (true)
         let expected = "Filter: Boolean(true) AS CAST(now() AS Int64) < CAST(totimestamp(Utf8(\"2020-09-08T12:05:00+00:00\")) AS Int64) + Int32(50000)\
-                        \n  TableScan: test projection=None";
+                        \n  TableScan: test";
         let actual = get_optimized_plan_formatted(&plan, &time);
 
         assert_eq!(expected, actual);
@@ -1964,7 +1959,7 @@ mod tests {
         // Note that constant folder runs and folds the entire
         // expression down to a single constant (true)
         let expected = "Projection: Date32(\"18636\") AS CAST(totimestamp(Utf8(\"2020-09-08T12:05:00+00:00\")) AS Date32) + IntervalDayTime(\"123\")\
-            \n  TableScan: test projection=None";
+            \n  TableScan: test";
         let actual = get_optimized_plan_formatted(&plan, &time);
 
         assert_eq!(expected, actual);
