@@ -336,8 +336,8 @@ mod tests {
         let b: ArrayRef = Arc::new(Int32Array::from(vec![Some(4), Some(5), Some(6)]));
 
         let schema = Schema::new(vec![
-            Field::new("a", DataType::Int32, false),
-            Field::new("b", DataType::Int32, false),
+            Field::new("a", DataType::Int32, true),
+            Field::new("b", DataType::Int32, true),
         ]);
         let batch = RecordBatch::try_new(Arc::new(schema.clone()), vec![a, b])?;
 
@@ -359,8 +359,8 @@ mod tests {
         let b: ArrayRef = Arc::new(Int32Array::from(vec![None, None]));
 
         let schema = Schema::new(vec![
-            Field::new("a", DataType::Int32, false),
-            Field::new("b", DataType::Int32, false),
+            Field::new("a", DataType::Int32, true),
+            Field::new("b", DataType::Int32, true),
         ]);
         let batch = RecordBatch::try_new(Arc::new(schema.clone()), vec![a, b])?;
 
@@ -384,8 +384,8 @@ mod tests {
         let d = Arc::new(Float64Array::from(vec![4.4_f64, 5.5_f64, 9.9_f64]));
 
         let schema = Schema::new(vec![
-            Field::new("a", DataType::Float64, false),
-            Field::new("b", DataType::Float64, false),
+            Field::new("a", DataType::Float64, true),
+            Field::new("b", DataType::Float64, true),
         ]);
 
         let batch1 = RecordBatch::try_new(Arc::new(schema.clone()), vec![a, b])?;
@@ -419,8 +419,8 @@ mod tests {
         let d = Arc::new(Float64Array::from(vec![None]));
 
         let schema = Schema::new(vec![
-            Field::new("a", DataType::Float64, false),
-            Field::new("b", DataType::Float64, false),
+            Field::new("a", DataType::Float64, true),
+            Field::new("b", DataType::Float64, true),
         ]);
 
         let batch1 = RecordBatch::try_new(Arc::new(schema.clone()), vec![a, b])?;
