@@ -511,8 +511,8 @@ mod tests {
         let b: ArrayRef = Arc::new(Int32Array::from(vec![Some(4), Some(5), Some(6)]));
 
         let schema = Schema::new(vec![
-            Field::new("a", DataType::Int32, false),
-            Field::new("b", DataType::Int32, false),
+            Field::new("a", DataType::Int32, true),
+            Field::new("b", DataType::Int32, true),
         ]);
         let batch = RecordBatch::try_new(Arc::new(schema.clone()), vec![a, b])?;
 
@@ -534,8 +534,8 @@ mod tests {
         let b: ArrayRef = Arc::new(Int32Array::from(vec![None, None]));
 
         let schema = Schema::new(vec![
-            Field::new("a", DataType::Int32, false),
-            Field::new("b", DataType::Int32, false),
+            Field::new("a", DataType::Int32, true),
+            Field::new("b", DataType::Int32, true),
         ]);
         let batch = RecordBatch::try_new(Arc::new(schema.clone()), vec![a, b])?;
 
@@ -559,8 +559,8 @@ mod tests {
         let d = Arc::new(Float64Array::from(vec![4.4_f64, 5.5_f64, 6.6_f64]));
 
         let schema = Schema::new(vec![
-            Field::new("a", DataType::Float64, false),
-            Field::new("b", DataType::Float64, false),
+            Field::new("a", DataType::Float64, true),
+            Field::new("b", DataType::Float64, true),
         ]);
 
         let batch1 = RecordBatch::try_new(Arc::new(schema.clone()), vec![a, b])?;
@@ -594,8 +594,8 @@ mod tests {
         let d = Arc::new(Float64Array::from(vec![None]));
 
         let schema = Schema::new(vec![
-            Field::new("a", DataType::Float64, false),
-            Field::new("b", DataType::Float64, false),
+            Field::new("a", DataType::Float64, true),
+            Field::new("b", DataType::Float64, true),
         ]);
 
         let batch1 = RecordBatch::try_new(Arc::new(schema.clone()), vec![a, b])?;
