@@ -26,8 +26,8 @@ use arrow::compute;
 use arrow::datatypes::DataType;
 use arrow::{
     array::{
-        ArrayRef, Float32Array, Float64Array, Int16Array, Int32Array, Int64Array,
-        Int8Array, UInt16Array, UInt32Array, UInt64Array, UInt8Array,
+        ArrayRef, BasicDecimalArray, Float32Array, Float64Array, Int16Array, Int32Array,
+        Int64Array, Int8Array, UInt16Array, UInt32Array, UInt64Array, UInt8Array,
     },
     datatypes::Field,
 };
@@ -166,7 +166,7 @@ fn sum_decimal_batch(
     let mut result = 0_i128;
     for i in 0..array.len() {
         if array.is_valid(i) {
-            result += array.value(i);
+            result += array.value(i).as_i128();
         }
     }
     Ok(ScalarValue::Decimal128(Some(result), *precision, *scale))
