@@ -244,7 +244,7 @@ mod tests {
     #[test]
     fn get_indexed_field_invalid_scalar() -> Result<()> {
         let schema = list_schema("l");
-        let expr = lit(ScalarValue::Utf8(Some("a".to_string())));
+        let expr = lit("a");
         get_indexed_field_test_failure(schema, expr,  ScalarValue::Int64(Some(0)), "Execution error: get indexed field is only possible on lists with int64 indexes or struct with utf8 indexes. Tried Utf8 with Int64(0) index")
     }
 
