@@ -119,8 +119,10 @@ mod tests {
     use super::*;
     use crate::expressions::col;
     use arrow::array::{
-        DecimalArray, DecimalBuilder, StringArray, Time64NanosecondArray,
+        BasicDecimalArray, DecimalArray, DecimalBuilder, StringArray,
+        Time64NanosecondArray,
     };
+    use arrow::util::decimal::{BasicDecimal, Decimal128};
     use arrow::{
         array::{
             Array, Float32Array, Float64Array, Int16Array, Int32Array, Int64Array,
@@ -138,7 +140,7 @@ mod tests {
     // 5. verify that the resulting values are downcastable and correct
     macro_rules! generic_decimal_to_other_test_cast {
         ($DECIMAL_ARRAY:ident, $A_TYPE:expr, $TYPEARRAY:ident, $TYPE:expr, $VEC:expr) => {{
-            let schema = Schema::new(vec![Field::new("a", $A_TYPE, false)]);
+            let schema = Schema::new(vec![Field::new("a", $A_TYPE, true)]);
             let batch = RecordBatch::try_new(
                 Arc::new(schema.clone()),
                 vec![Arc::new($DECIMAL_ARRAY)],
@@ -185,7 +187,7 @@ mod tests {
     // 5. verify that the resulting values are downcastable and correct
     macro_rules! generic_test_cast {
         ($A_ARRAY:ident, $A_TYPE:expr, $A_VEC:expr, $TYPEARRAY:ident, $TYPE:expr, $VEC:expr) => {{
-            let schema = Schema::new(vec![Field::new("a", $A_TYPE, false)]);
+            let schema = Schema::new(vec![Field::new("a", $A_TYPE, true)]);
             let a = $A_ARRAY::from($A_VEC);
             let batch =
                 RecordBatch::try_new(Arc::new(schema.clone()), vec![Arc::new(a)])?;
@@ -232,33 +234,35 @@ mod tests {
         // try cast one decimal data type to another decimal data type
         let array: Vec<i128> = vec![1234, 2222, 3, 4000, 5000];
         let decimal_array = create_decimal_array(&array, 10, 3)?;
+        let convert = |v: i128| Decimal128::new(20, 6, &v.to_le_bytes());
         generic_decimal_to_other_test_cast!(
             decimal_array,
             DataType::Decimal(10, 3),
             DecimalArray,
             DataType::Decimal(20, 6),
             vec![
-                Some(1_234_000_i128),
-                Some(2_222_000_i128),
-                Some(3_000_i128),
-                Some(4_000_000_i128),
-                Some(5_000_000_i128),
+                Some(convert(1_234_000)),
+                Some(convert(2_222_000)),
+                Some(convert(3_000)),
+                Some(convert(4_000_000)),
+                Some(convert(5_000_000)),
                 None,
             ]
         );
 
         let decimal_array = create_decimal_array(&array, 10, 3)?;
+        let convert = |v: i128| Decimal128::new(10, 2, &v.to_le_bytes());
         generic_decimal_to_other_test_cast!(
             decimal_array,
             DataType::Decimal(10, 3),
             DecimalArray,
             DataType::Decimal(10, 2),
             vec![
-                Some(123_i128),
-                Some(222_i128),
-                Some(0_i128),
-                Some(400_i128),
-                Some(500_i128),
+                Some(convert(123)),
+                Some(convert(222)),
+                Some(convert(0)),
+                Some(convert(400)),
+                Some(convert(500)),
                 None,
             ]
         );
@@ -379,6 +383,7 @@ mod tests {
     #[test]
     fn test_try_cast_numeric_to_decimal() -> Result<()> {
         // int8
+        let convert = |v: i128| Decimal128::new(3, 0, &v.to_le_bytes());
         generic_test_cast!(
             Int8Array,
             DataType::Int8,
@@ -386,15 +391,16 @@ mod tests {
             DecimalArray,
             DataType::Decimal(3, 0),
             vec![
-                Some(1_i128),
-                Some(2_i128),
-                Some(3_i128),
-                Some(4_i128),
-                Some(5_i128),
+                Some(convert(1)),
+                Some(convert(2)),
+                Some(convert(3)),
+                Some(convert(4)),
+                Some(convert(5)),
             ]
         );
 
         // int16
+        let convert = |v: i128| Decimal128::new(5, 0, &v.to_le_bytes());
         generic_test_cast!(
             Int16Array,
             DataType::Int16,
@@ -402,15 +408,16 @@ mod tests {
             DecimalArray,
             DataType::Decimal(5, 0),
             vec![
-                Some(1_i128),
-                Some(2_i128),
-                Some(3_i128),
-                Some(4_i128),
-                Some(5_i128),
+                Some(convert(1)),
+                Some(convert(2)),
+                Some(convert(3)),
+                Some(convert(4)),
+                Some(convert(5)),
             ]
         );
 
         // int32
+        let convert = |v: i128| Decimal128::new(10, 0, &v.to_le_bytes());
         generic_test_cast!(
             Int32Array,
             DataType::Int32,
@@ -418,15 +425,16 @@ mod tests {
             DecimalArray,
             DataType::Decimal(10, 0),
             vec![
-                Some(1_i128),
-                Some(2_i128),
-                Some(3_i128),
-                Some(4_i128),
-                Some(5_i128),
+                Some(convert(1)),
+                Some(convert(2)),
+                Some(convert(3)),
+                Some(convert(4)),
+                Some(convert(5)),
             ]
         );
 
         // int64
+        let convert = |v: i128| Decimal128::new(20, 0, &v.to_le_bytes());
         generic_test_cast!(
             Int64Array,
             DataType::Int64,
@@ -434,15 +442,16 @@ mod tests {
             DecimalArray,
             DataType::Decimal(20, 0),
             vec![
-                Some(1_i128),
-                Some(2_i128),
-                Some(3_i128),
-                Some(4_i128),
-                Some(5_i128),
+                Some(convert(1)),
+                Some(convert(2)),
+                Some(convert(3)),
+                Some(convert(4)),
+                Some(convert(5)),
             ]
         );
 
         // int64 to different scale
+        let convert = |v: i128| Decimal128::new(20, 2, &v.to_le_bytes());
         generic_test_cast!(
             Int64Array,
             DataType::Int64,
@@ -450,15 +459,16 @@ mod tests {
             DecimalArray,
             DataType::Decimal(20, 2),
             vec![
-                Some(100_i128),
-                Some(200_i128),
-                Some(300_i128),
-                Some(400_i128),
-                Some(500_i128),
+                Some(convert(100)),
+                Some(convert(200)),
+                Some(convert(300)),
+                Some(convert(400)),
+                Some(convert(500)),
             ]
         );
 
         // float32
+        let convert = |v: i128| Decimal128::new(10, 2, &v.to_le_bytes());
         generic_test_cast!(
             Float32Array,
             DataType::Float32,
@@ -466,15 +476,16 @@ mod tests {
             DecimalArray,
             DataType::Decimal(10, 2),
             vec![
-                Some(150_i128),
-                Some(250_i128),
-                Some(300_i128),
-                Some(112_i128),
-                Some(550_i128),
+                Some(convert(150)),
+                Some(convert(250)),
+                Some(convert(300)),
+                Some(convert(112)),
+                Some(convert(550)),
             ]
         );
 
         // float64
+        let convert = |v: i128| Decimal128::new(20, 4, &v.to_le_bytes());
         generic_test_cast!(
             Float64Array,
             DataType::Float64,
@@ -482,11 +493,11 @@ mod tests {
             DecimalArray,
             DataType::Decimal(20, 4),
             vec![
-                Some(15000_i128),
-                Some(25000_i128),
-                Some(30000_i128),
-                Some(11234_i128),
-                Some(55000_i128),
+                Some(convert(15000)),
+                Some(convert(25000)),
+                Some(convert(30000)),
+                Some(convert(11234)),
+                Some(convert(55000)),
             ]
         );
         Ok(())
