@@ -20,12 +20,17 @@ use crate::protobuf::plan_type::PlanTypeEnum::{
     FinalLogicalPlan, FinalPhysicalPlan, InitialLogicalPlan, InitialPhysicalPlan,
     OptimizedLogicalPlan, OptimizedPhysicalPlan,
 };
-use crate::protobuf::{OptimizedLogicalPlanType, OptimizedPhysicalPlanType};
+use crate::protobuf::{
+    CubeNode, GroupingSetNode, OptimizedLogicalPlanType, OptimizedPhysicalPlanType,
+    RollupNode,
+};
 use arrow::datatypes::{DataType, Field, IntervalUnit, Schema, TimeUnit, UnionMode};
 use datafusion::logical_plan::FunctionRegistry;
 use datafusion_common::{
     Column, DFField, DFSchema, DFSchemaRef, DataFusionError, ScalarValue,
 };
+use datafusion_expr::expr::GroupingSet;
+use datafusion_expr::expr::GroupingSet::GroupingSets;
 use datafusion_expr::{
     abs, acos, array, ascii, asin, atan, bit_length, btrim, ceil, character_length, chr,
     coalesce, concat_expr, concat_ws_expr, cos, date_part, date_trunc, digest, exp,
@@ -1290,6 +1295,32 @@ pub fn parse_expr(
                     .collect::<Result<Vec<_>, Error>>()?,
             })
         }
+
+        ExprType::GroupingSet(GroupingSetNode { expr }) => {
+            Ok(Expr::GroupingSet(GroupingSets(
+                expr.iter()
+                    .map(|expr_list| {
+                        expr_list
+                            .expr
+                            .iter()
+                            .map(|expr| parse_expr(expr, registry))
+                            .collect::<Result<Vec<_>, Error>>()
+                    })
+                    .collect::<Result<Vec<_>, Error>>()?,
+            )))
+        }
+        ExprType::Cube(CubeNode { expr }) => Ok(Expr::GroupingSet(GroupingSet::Cube(
+            expr.iter()
+                .map(|expr| parse_expr(expr, registry))
+                .collect::<Result<Vec<_>, Error>>()?,
+        ))),
+        ExprType::Rollup(RollupNode { expr }) => {
+            Ok(Expr::GroupingSet(GroupingSet::Rollup(
+                expr.iter()
+                    .map(|expr| parse_expr(expr, registry))
+                    .collect::<Result<Vec<_>, Error>>()?,
+            )))
+        }
     }
 }
 
