@@ -35,17 +35,11 @@ crates = {
     'datafusion-data-access': 'datafusion/data-access/Cargo.toml',
     'datafusion-expr': 'datafusion/expr/Cargo.toml',
     'datafusion-jit': 'datafusion/jit/Cargo.toml',
+    'datafusion-optimizer': 'datafusion/optimizer/Cargo.toml',
     'datafusion-physical-expr': 'datafusion/physical-expr/Cargo.toml',
     'datafusion-proto': 'datafusion/proto/Cargo.toml',
-    'datafusion-row': 'datafusion/row/Cargo.toml'
-}
-
-ballista_crates = {
-    'ballista-cli': 'ballista-cli/Cargo.toml',
-    'core': 'ballista/rust/core/Cargo.toml',
-    'client': 'ballista/rust/client/Cargo.toml',
-    'executor': 'ballista/rust/executor/Cargo.toml',
-    'scheduler': 'ballista/rust/scheduler/Cargo.toml',
+    'datafusion-row': 'datafusion/row/Cargo.toml',
+    'datafusion-sql': 'datafusion/sql/Cargo.toml',
 }
 
 def update_datafusion_version(cargo_toml: str, new_version: str):
@@ -110,8 +104,6 @@ def main():
     print(f'Updating datafusion dependency versions in {repo_root} to {new_version}')
     for cargo_toml in crates.values():
         update_downstream_versions(cargo_toml, new_version)
-    for cargo_toml in ballista_crates.values():
-        update_downstream_versions(cargo_toml, new_version)
 
     update_docs("README.md", new_version)
 
